import { A11yModule } from '@angular/cdk/a11y';
import { ComponentPortal, CdkPortalOutlet, PortalModule } from '@angular/cdk/portal';
import { CommonModule } from '@angular/common';
import { EventEmitter, Component, ViewEncapsulation, ChangeDetectionStrategy, ElementRef, NgZone, Input, Output, Directive, Injectable, Optional, SkipSelf, InjectionToken, ChangeDetectorRef, Inject, ViewChild, forwardRef, InjectFlags, Injector, Attribute, Self, ContentChild, NgModule } from '@angular/core';
import { DateAdapter, NT_DATE_FORMATS, fadeOut, fadeIn, NtPseudoInputModule } from '@ng-tangram/components/core';
import { BOTTOM_LEFT, TOP_LEFT, NtOverlayComponent, NtOverlayModule } from '@ng-tangram/components/overlay';
import { take, startWith, debounceTime } from 'rxjs/operators';
import { Subject, Subscription } from 'rxjs';
import { FormControl, NgControl, Validators, NG_VALUE_ACCESSOR, NG_VALIDATORS, ControlContainer } from '@angular/forms';
import { coerceBooleanProperty } from '@angular/cdk/coercion';
import { trigger, transition } from '@angular/animations';
import { NtFormFieldControl } from '@ng-tangram/components/forms';
import { CdkOverlayOrigin } from '@angular/cdk/overlay';

const yearsPerPage = 12;
const yearsPerRow = 3;
function isSameMultiYearView(dateAdapter, date1, date2, minDate, maxDate) {
    const year1 = dateAdapter.getYear(date1);
    const year2 = dateAdapter.getYear(date2);
    const startingYear = getStartingYear(dateAdapter, minDate, maxDate);
    return Math.floor((year1 - startingYear) / yearsPerPage) ===
        Math.floor((year2 - startingYear) / yearsPerPage);
}
function getActiveOffset(dateAdapter, activeDate, minDate, maxDate) {
    const activeYear = dateAdapter.getYear(activeDate);
    return euclideanModulo((activeYear - getStartingYear(dateAdapter, minDate, maxDate)), yearsPerPage);
}
function getStartingYear(dateAdapter, minDate, maxDate) {
    let startingYear = 0;
    if (maxDate) {
        const maxYear = dateAdapter.getYear(maxDate);
        startingYear = maxYear - yearsPerPage + 1;
    }
    else if (minDate) {
        startingYear = dateAdapter.getYear(minDate);
    }
    return startingYear;
}
function euclideanModulo(a, b) {
    return (a % b + b) % b;
}
function isTableCell(node) {
    return node.nodeName === 'TD';
}
function isStart(value, start, end) {
    return end !== null && start !== end && value < end && value === start;
}
function isEnd(value, start, end) {
    return start !== null && start !== end && value >= start && value === end;
}
function isInRange(value, start, end, rangeEnabled) {
    return rangeEnabled && start !== null && end !== null && start !== end &&
        value >= start && value <= end;
}

class NtCalendarCell {
    constructor(value, displayValue, enabled, cssClasses = {}, compareValue = value, rawValue) {
        this.value = value;
        this.displayValue = displayValue;
        this.enabled = enabled;
        this.cssClasses = cssClasses;
        this.compareValue = compareValue;
        this.rawValue = rawValue;
    }
}
class NtCalendarBody {
    constructor(_elementRef, _ngZone) {
        this._elementRef = _elementRef;
        this._ngZone = _ngZone;
        this.numCols = 7;
        this.activeCell = 0;
        this.isRange = false;
        this.cellAspectRatio = 1;
        this.previewStart = null;
        this.previewEnd = null;
        this.selectedValueChange = new EventEmitter();
        this.previewChange = new EventEmitter();
        this._enterHandler = (event) => {
            if (this._skipNextFocus && event.type === 'focus') {
                this._skipNextFocus = false;
                return;
            }
            if (event.target && this.isRange) {
                const cell = this._getCellFromElement(event.target);
                if (cell) {
                    this._ngZone.run(() => this.previewChange.emit({ value: cell.enabled ? cell : null, event }));
                }
            }
        };
        this._leaveHandler = (event) => {
            if (this.previewEnd !== null && this.isRange) {
                if (event.target && isTableCell(event.target)) {
                    this._ngZone.run(() => this.previewChange.emit({ value: null, event }));
                }
            }
        };
        _ngZone.runOutsideAngular(() => {
            const element = _elementRef.nativeElement;
            element.addEventListener('mouseenter', this._enterHandler, true);
            element.addEventListener('focus', this._enterHandler, true);
            element.addEventListener('mouseleave', this._leaveHandler, true);
            element.addEventListener('blur', this._leaveHandler, true);
        });
    }
    _cellClicked(cell, event) {
        if (cell.enabled) {
            this.selectedValueChange.emit({ value: cell.value, event });
        }
    }
    _isSelected(value) {
        return this.startValue === value || this.endValue === value;
    }
    ngOnChanges(changes) {
        const columnChanges = changes['numCols'];
        const { rows, numCols } = this;
        if (changes['rows'] || columnChanges) {
            this._firstRowOffset = rows && rows.length && rows[0].length ? numCols - rows[0].length : 0;
        }
        if (changes['cellAspectRatio'] || columnChanges || !this._cellPadding) {
            this._cellPadding = `${50 * this.cellAspectRatio / numCols}%`;
        }
        if (columnChanges || !this._cellWidth) {
            this._cellWidth = `${100 / numCols}%`;
        }
    }
    ngOnDestroy() {
        const element = this._elementRef.nativeElement;
        element.removeEventListener('mouseenter', this._enterHandler, true);
        element.removeEventListener('focus', this._enterHandler, true);
        element.removeEventListener('mouseleave', this._leaveHandler, true);
        element.removeEventListener('blur', this._leaveHandler, true);
    }
    _isActiveCell(rowIndex, colIndex) {
        let cellNumber = rowIndex * this.numCols + colIndex;
        if (rowIndex) {
            cellNumber -= this._firstRowOffset;
        }
        return cellNumber == this.activeCell;
    }
    _focusActiveCell(movePreview = true) {
        this._ngZone.runOutsideAngular(() => {
            this._ngZone.onStable.pipe(take(1)).subscribe(() => {
                const activeCell = this._elementRef.nativeElement.querySelector('.nt-calendar-body-active');
                if (activeCell) {
                    if (!movePreview) {
                        this._skipNextFocus = true;
                    }
                    activeCell.focus();
                }
            });
        });
    }
    _isRangeStart(value) {
        return isStart(value, this.startValue, this.endValue);
    }
    _isRangeEnd(value) {
        return isEnd(value, this.startValue, this.endValue);
    }
    _isInRange(value) {
        return isInRange(value, this.startValue, this.endValue, this.isRange);
    }
    _isComparisonStart(value) {
        return isStart(value, this.comparisonStart, this.comparisonEnd);
    }
    _isComparisonBridgeStart(value, rowIndex, colIndex) {
        if (!this._isComparisonStart(value) || this._isRangeStart(value) || !this._isInRange(value)) {
            return false;
        }
        let previousCell = this.rows[rowIndex][colIndex - 1];
        if (!previousCell) {
            const previousRow = this.rows[rowIndex - 1];
            previousCell = previousRow && previousRow[previousRow.length - 1];
        }
        return previousCell && !this._isRangeEnd(previousCell.compareValue);
    }
    _isComparisonBridgeEnd(value, rowIndex, colIndex) {
        if (!this._isComparisonEnd(value) || this._isRangeEnd(value) || !this._isInRange(value)) {
            return false;
        }
        let nextCell = this.rows[rowIndex][colIndex + 1];
        if (!nextCell) {
            const nextRow = this.rows[rowIndex + 1];
            nextCell = nextRow && nextRow[0];
        }
        return nextCell && !this._isRangeStart(nextCell.compareValue);
    }
    _isComparisonEnd(value) {
        return isEnd(value, this.comparisonStart, this.comparisonEnd);
    }
    _isInComparisonRange(value) {
        return isInRange(value, this.comparisonStart, this.comparisonEnd, this.isRange);
    }
    _isComparisonIdentical(value) {
        return this.comparisonStart === this.comparisonEnd && value === this.comparisonStart;
    }
    _isPreviewStart(value) {
        return isStart(value, this.previewStart, this.previewEnd);
    }
    _isPreviewEnd(value) {
        return isEnd(value, this.previewStart, this.previewEnd);
    }
    _isInPreview(value) {
        return isInRange(value, this.previewStart, this.previewEnd, this.isRange);
    }
    _getCellFromElement(element) {
        let cell;
        if (isTableCell(element)) {
            cell = element;
        }
        else if (isTableCell(element.parentNode)) {
            cell = element.parentNode;
        }
        if (cell) {
            const row = cell.getAttribute('data-nt-row');
            const col = cell.getAttribute('data-nt-col');
            if (row && col) {
                return this.rows[parseInt(row)][parseInt(col)];
            }
        }
        return null;
    }
}
NtCalendarBody.decorators = [
    { type: Component, args: [{
                selector: '[nt-calendar-body]',
                template: "<!-- Create the first row separately so we can include a special spacer cell. -->\n<tr *ngFor=\"let row of rows; let rowIndex = index\" role=\"row\">\n  <td *ngFor=\"let item of row; let colIndex = index\"\n      role=\"gridcell\"\n      class=\"nt-calendar-cell\"\n      [ngClass]=\"item.cssClasses\"\n      [tabindex]=\"_isActiveCell(rowIndex, colIndex) ? 0 : -1\"\n      [attr.data-nt-row]=\"rowIndex\"\n      [attr.data-nt-col]=\"colIndex\"\n      [class.nt-calendar-cell-disabled]=\"!item.enabled\"\n      [class.nt-calendar-cell-active]=\"_isActiveCell(rowIndex, colIndex)\"\n      [class.nt-calendar-cell-range-start]=\"_isRangeStart(item.compareValue)\"\n      [class.nt-calendar-cell-range-end]=\"_isRangeEnd(item.compareValue)\"\n      [class.nt-calendar-cell-in-range]=\"_isInRange(item.compareValue)\"\n      [class.nt-calendar-cell-comparison-bridge-start]=\"_isComparisonBridgeStart(item.compareValue, rowIndex, colIndex)\"\n      [class.nt-calendar-cell-comparison-bridge-end]=\"_isComparisonBridgeEnd(item.compareValue, rowIndex, colIndex)\"\n      [class.nt-calendar-cell-comparison-start]=\"_isComparisonStart(item.compareValue)\"\n      [class.nt-calendar-cell-comparison-end]=\"_isComparisonEnd(item.compareValue)\"\n      [class.nt-calendar-cell-in-comparison-range]=\"_isInComparisonRange(item.compareValue)\"\n      [class.nt-calendar-cell-preview-start]=\"_isPreviewStart(item.compareValue)\"\n      [class.nt-calendar-cell-preview-end]=\"_isPreviewEnd(item.compareValue)\"\n      [class.nt-calendar-cell-in-preview]=\"_isInPreview(item.compareValue)\"\n      (click)=\"_cellClicked(item, $event)\">\n      <span class=\"nt-calendar-cell-content nt-focus-indicator\"\n        [class.nt-calendar-cell-selected]=\"_isSelected(item.compareValue)\"\n        [class.nt-calendar-cell-comparison-identical]=\"_isComparisonIdentical(item.compareValue)\"\n        [class.nt-calendar-cell-today]=\"todayValue === item.compareValue\">\n        {{item.displayValue}}\n      </span>\n      <div class=\"nt-calendar-body-cell-preview\"></div>\n  </td>\n</tr>\n",
                host: {
                    'class': 'nt-calendar-body'
                },
                exportAs: 'ntCalendarBody',
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] }
];
NtCalendarBody.ctorParameters = () => [
    { type: ElementRef },
    { type: NgZone }
];
NtCalendarBody.propDecorators = {
    label: [{ type: Input }],
    rows: [{ type: Input }],
    todayValue: [{ type: Input }],
    startValue: [{ type: Input }],
    endValue: [{ type: Input }],
    numCols: [{ type: Input }],
    activeCell: [{ type: Input }],
    isRange: [{ type: Input }],
    cellAspectRatio: [{ type: Input }],
    comparisonStart: [{ type: Input }],
    comparisonEnd: [{ type: Input }],
    previewStart: [{ type: Input }],
    previewEnd: [{ type: Input }],
    selectedValueChange: [{ type: Output }],
    previewChange: [{ type: Output }]
};

class DateRange {
    constructor(start, end) {
        this.start = start;
        this.end = end;
    }
}
class NtDateSelectionModel {
    constructor(selection, _adapter) {
        this.selection = selection;
        this._adapter = _adapter;
        this._selectionChanged = new Subject();
        this.selectionChanged = this._selectionChanged;
        this.selection = selection;
    }
    updateSelection(value, source) {
        this.selection = value;
        this._selectionChanged.next({ selection: value, source });
    }
    ngOnDestroy() {
        this._selectionChanged.complete();
    }
    _isValidDateInstance(date) {
        return this._adapter.isDateInstance(date) && this._adapter.isValid(date);
    }
}
NtDateSelectionModel.decorators = [
    { type: Directive }
];
NtDateSelectionModel.ctorParameters = () => [
    { type: undefined },
    { type: DateAdapter }
];
class NtSingleDateSelectionModel extends NtDateSelectionModel {
    constructor(adapter) {
        super(null, adapter);
    }
    add(date) {
        super.updateSelection(date, this);
    }
    isValid() {
        return this.selection != null && this._isValidDateInstance(this.selection);
    }
    isComplete() {
        return this.selection != null;
    }
}
NtSingleDateSelectionModel.decorators = [
    { type: Injectable }
];
NtSingleDateSelectionModel.ctorParameters = () => [
    { type: DateAdapter }
];
class NtRangeDateSelectionModel extends NtDateSelectionModel {
    constructor(adapter) {
        super(new DateRange(null, null), adapter);
    }
    add(date) {
        let { start, end } = this.selection;
        if (start == null) {
            start = date;
        }
        else if (end == null) {
            end = date;
        }
        else {
            start = date;
            end = null;
        }
        super.updateSelection(new DateRange(start, end), this);
    }
    isValid() {
        const { start, end } = this.selection;
        if (start == null && end == null) {
            return true;
        }
        if (start != null && end != null) {
            return this._isValidDateInstance(start) && this._isValidDateInstance(end) &&
                this._adapter.compareDate(start, end) <= 0;
        }
        return (start == null || this._isValidDateInstance(start)) &&
            (end == null || this._isValidDateInstance(end));
    }
    isComplete() {
        return this.selection.start != null && this.selection.end != null;
    }
}
NtRangeDateSelectionModel.decorators = [
    { type: Injectable }
];
NtRangeDateSelectionModel.ctorParameters = () => [
    { type: DateAdapter }
];
function NT_SINGLE_DATE_SELECTION_MODEL_FACTORY(parent, adapter) {
    return parent || new NtSingleDateSelectionModel(adapter);
}
const NT_SINGLE_DATE_SELECTION_MODEL_PROVIDER = {
    provide: NtDateSelectionModel,
    deps: [[new Optional(), new SkipSelf(), NtDateSelectionModel], DateAdapter],
    useFactory: NT_SINGLE_DATE_SELECTION_MODEL_FACTORY,
};
function NT_RANGE_DATE_SELECTION_MODEL_FACTORY(parent, adapter) {
    return parent || new NtRangeDateSelectionModel(adapter);
}
const NT_RANGE_DATE_SELECTION_MODEL_PROVIDER = {
    provide: NtDateSelectionModel,
    deps: [[new Optional(), new SkipSelf(), NtDateSelectionModel], DateAdapter],
    useFactory: NT_RANGE_DATE_SELECTION_MODEL_FACTORY,
};

const NT_DATE_RANGE_SELECTION_STRATEGY = new InjectionToken('NT_DATE_RANGE_SELECTION_STRATEGY');
class DefaultNtCalendarRangeStrategy {
    constructor(_dateAdapter) {
        this._dateAdapter = _dateAdapter;
    }
    selectionFinished(date, currentRange) {
        let { start, end } = currentRange;
        if (start == null) {
            start = date;
        }
        else if (end == null && date) {
            if (this._dateAdapter.compareDate(date, start) >= 0) {
                end = date;
            }
            else {
                start = date;
                end = currentRange.start;
            }
        }
        else {
            start = date;
            end = null;
        }
        return new DateRange(start, end);
    }
    createPreview(activeDate, currentRange) {
        let start = null;
        let end = null;
        if (currentRange.start && !currentRange.end && activeDate) {
            start = currentRange.start;
            end = activeDate;
            if (this._dateAdapter.compareDate(activeDate, start) < 0) {
                start = activeDate;
                end = currentRange.start;
            }
        }
        return new DateRange(start, end);
    }
}
DefaultNtCalendarRangeStrategy.decorators = [
    { type: Injectable }
];
DefaultNtCalendarRangeStrategy.ctorParameters = () => [
    { type: DateAdapter }
];
function NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY(parent, adapter) {
    return parent || new DefaultNtCalendarRangeStrategy(adapter);
}
const NT_CALENDAR_RANGE_STRATEGY_PROVIDER = {
    provide: NT_DATE_RANGE_SELECTION_STRATEGY,
    deps: [[new Optional(), new SkipSelf(), NT_DATE_RANGE_SELECTION_STRATEGY], DateAdapter],
    useFactory: NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY,
};

class WeeklyNtCalendarRangeStrategy {
    constructor(_dateAdapter) {
        this._dateAdapter = _dateAdapter;
    }
    selectionFinished(date) {
        if (date) {
            return this._getRangeRelativeToDate(date);
        }
        return new DateRange(null, null);
    }
    createPreview(activeDate) {
        if (activeDate) {
            return this._getRangeRelativeToDate(activeDate);
        }
        return new DateRange(null, null);
    }
    _getRangeRelativeToDate(date) {
        let rangeStart = null;
        let rangeEnd = null;
        if (date) {
            const firstDayOfWeek = this._dateAdapter.getFirstDayOfWeek();
            const dayOfWeek = this._dateAdapter.getDayOfWeek(date);
            rangeStart = this._dateAdapter.addCalendarDays(date, firstDayOfWeek - dayOfWeek);
            rangeEnd = this._dateAdapter.addCalendarDays(date, 6 - dayOfWeek);
        }
        return new DateRange(rangeStart, rangeEnd);
    }
}
WeeklyNtCalendarRangeStrategy.decorators = [
    { type: Injectable }
];
WeeklyNtCalendarRangeStrategy.ctorParameters = () => [
    { type: DateAdapter }
];
function WEEKLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY(parent, adapter) {
    return parent || new WeeklyNtCalendarRangeStrategy(adapter);
}
const WEEKLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER = {
    provide: NT_DATE_RANGE_SELECTION_STRATEGY,
    deps: [[new Optional(), new SkipSelf(), NT_DATE_RANGE_SELECTION_STRATEGY], DateAdapter],
    useFactory: WEEKLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY,
};

class MonthlyNtCalendarRangeStrategy {
    constructor(_dateAdapter) {
        this._dateAdapter = _dateAdapter;
    }
    selectionFinished(date) {
        if (date) {
            return this._getRangeRelativeToDate(date);
        }
        return new DateRange(null, null);
    }
    createPreview(activeDate) {
        if (activeDate) {
            return this._getRangeRelativeToDate(activeDate);
        }
        return new DateRange(null, null);
    }
    _getRangeRelativeToDate(date) {
        let rangeStart = null;
        let rangeEnd = null;
        if (date) {
            const year = this._dateAdapter.getYear(date);
            const month = this._dateAdapter.getMonth(date);
            const monthDays = this._dateAdapter.getNumDaysInMonth(date);
            rangeStart = this._dateAdapter.createDate(year, month, 1);
            rangeEnd = this._dateAdapter.createDate(year, month, monthDays);
        }
        return new DateRange(rangeStart, rangeEnd);
    }
}
MonthlyNtCalendarRangeStrategy.decorators = [
    { type: Injectable }
];
MonthlyNtCalendarRangeStrategy.ctorParameters = () => [
    { type: DateAdapter }
];
function MONTHLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY(parent, adapter) {
    return parent || new MonthlyNtCalendarRangeStrategy(adapter);
}
const MONTHLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER = {
    provide: NT_DATE_RANGE_SELECTION_STRATEGY,
    deps: [[new Optional(), new SkipSelf(), NT_DATE_RANGE_SELECTION_STRATEGY], DateAdapter],
    useFactory: MONTHLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY,
};

function createMissingDateImplError(provider) {
    return Error(`NtDatepickerComponent: No provider found for ${provider}. You must import one of the following ` +
        `modules at your application root: NtNativeDateModule or provide a ` +
        `custom implementation.`);
}

const DAYS_PER_WEEK = 7;
const DAYS_MAX_ROWS = 6;
class NtCalendarMonth {
    constructor(_changeDetectorRef, _dateAdapter, _dateFormats, _rangeStrategy) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        this._rangeStrategy = _rangeStrategy;
        this._rerenderSubscription = Subscription.EMPTY;
        this.selectedChange = new EventEmitter();
        this._userSelection = new EventEmitter();
        this.activeDateChange = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        if (!this._dateFormats) {
            throw createMissingDateImplError('NT_DATE_FORMATS');
        }
        const firstDayOfWeek = this._dateAdapter.getFirstDayOfWeek();
        const narrowWeekdays = this._dateAdapter.getDayOfWeekNames('narrow');
        const longWeekdays = this._dateAdapter.getDayOfWeekNames('long');
        let weekdays = longWeekdays.map((long, i) => {
            return { long, narrow: narrowWeekdays[i] };
        });
        this._weekdays = weekdays.slice(firstDayOfWeek).concat(weekdays.slice(0, firstDayOfWeek));
        this._activeDate = this._dateAdapter.today();
    }
    get activeDate() { return this._activeDate; }
    set activeDate(value) {
        const oldActiveDate = this._activeDate;
        const validDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value)) || this._dateAdapter.today();
        this._activeDate = this._dateAdapter.clampDate(validDate, this.minDate, this.maxDate);
        if (!this._hasSameMonthAndYear(oldActiveDate, this._activeDate)) {
            this._init();
        }
    }
    get selected() { return this._selected; }
    set selected(value) {
        if (value instanceof DateRange) {
            this._selected = value;
        }
        else {
            this._selected = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
        }
        this._setRanges(this._selected);
    }
    get minDate() { return this._minDate; }
    set minDate(value) {
        this._minDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get maxDate() { return this._maxDate; }
    set maxDate(value) {
        this._maxDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    ngAfterContentInit() {
        this._rerenderSubscription = this._dateAdapter.localeChanges
            .pipe(startWith(null))
            .subscribe(() => this._init());
    }
    ngOnChanges(changes) {
        const comparisonChange = changes['comparisonStart'] || changes['comparisonEnd'];
        if (comparisonChange && !comparisonChange.firstChange) {
            this._setRanges(this.selected);
        }
    }
    ngOnDestroy() {
        this._rerenderSubscription.unsubscribe();
    }
    _dateSelected(event) {
        const date = event.value;
        const selectedYear = this._dateAdapter.getYear(this.activeDate);
        const selectedMonth = this._dateAdapter.getMonth(this.activeDate);
        const selectedDate = this._dateAdapter.createDate(selectedYear, selectedMonth, date);
        let rangeStartDate;
        let rangeEndDate;
        if (this._selected instanceof DateRange) {
            rangeStartDate = this._getDateInCurrentMonth(this._selected.start);
            rangeEndDate = this._getDateInCurrentMonth(this._selected.end);
        }
        else {
            rangeStartDate = rangeEndDate = this._getDateInCurrentMonth(this._selected);
        }
        if (rangeStartDate !== date || rangeEndDate !== date) {
            this.selectedChange.emit(selectedDate);
        }
        this._userSelection.emit({ value: selectedDate, event: event.event });
    }
    _init() {
        this._setRanges(this.selected);
        this._todayDate = this._getCellCompareValue(this._dateAdapter.today());
        this._monthLabel =
            this._dateAdapter.getMonthNames('short')[this._dateAdapter.getMonth(this.activeDate)]
                .toLocaleUpperCase();
        let firstOfMonth = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), this._dateAdapter.getMonth(this.activeDate), 1);
        this._firstWeekOffset =
            (DAYS_PER_WEEK + this._dateAdapter.getDayOfWeek(firstOfMonth) -
                this._dateAdapter.getFirstDayOfWeek()) % DAYS_PER_WEEK;
        this._initWeekdays();
        this._createWeekCells();
        this._changeDetectorRef.markForCheck();
    }
    _focusActiveCell(movePreview) {
        this._calendarBody._focusActiveCell(movePreview);
    }
    _previewChanged({ event, value: cell }) {
        if (this._rangeStrategy) {
            const value = cell ? cell.rawValue : null;
            const previewRange = this._rangeStrategy.createPreview(value, this.selected, event);
            this._previewStart = this._getCellCompareValue(previewRange.start);
            this._previewEnd = this._getCellCompareValue(previewRange.end);
            this._changeDetectorRef.detectChanges();
        }
    }
    _initWeekdays() {
        const firstDayOfWeek = this._dateAdapter.getFirstDayOfWeek();
        const narrowWeekdays = this._dateAdapter.getDayOfWeekNames('narrow');
        const longWeekdays = this._dateAdapter.getDayOfWeekNames('long');
        let weekdays = longWeekdays.map((long, i) => {
            return { long, narrow: narrowWeekdays[i] };
        });
        this._weekdays = weekdays.slice(firstDayOfWeek).concat(weekdays.slice(0, firstDayOfWeek));
    }
    _createWeekCells() {
        const daysInMonth = this._dateAdapter.getNumDaysInMonth(this.activeDate);
        const dateNames = this._dateAdapter.getDateNames();
        this._weeks = [[]];
        for (let i = 0, cell = this._firstWeekOffset; i < daysInMonth; i++, cell++) {
            if (cell == DAYS_PER_WEEK) {
                this._weeks.push([]);
                cell = 0;
            }
            const date = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), this._dateAdapter.getMonth(this.activeDate), i + 1);
            const enabled = this._shouldEnableDate(date);
            const cellClasses = this.dateClass ? this.dateClass(date, 'month') : undefined;
            this._weeks[this._weeks.length - 1].push(new NtCalendarCell(i + 1, dateNames[i], enabled, cellClasses, this._getCellCompareValue(date), date));
        }
        this._fillBeforeCells();
        this._fillAfterCells();
    }
    _fillBeforeCells() {
        const beforeMonth = this._dateAdapter.addCalendarMonths(this._dateAdapter.clone(this.activeDate), -1);
        const daysInBeforeMonth = this._dateAdapter.getNumDaysInMonth(beforeMonth);
        const dateNames = this._dateAdapter.getDateNames();
        const beforeWeeks = [];
        for (let i = daysInBeforeMonth - this._firstWeekOffset; i < daysInBeforeMonth; i++) {
            beforeWeeks.push(new NtCalendarCell(i + 1, dateNames[i], false));
        }
        this._weeks[0] = beforeWeeks.concat(this._weeks[0]);
    }
    _fillAfterCells() {
        const afterMonth = this._dateAdapter.addCalendarMonths(this._dateAdapter.clone(this.activeDate), 1);
        const lastRow = this._weeks[this._weeks.length - 1];
        const dateNames = this._dateAdapter.getDateNames();
        const afterWeeks = [], afterRows = [];
        for (let i = 0; i < DAYS_PER_WEEK - lastRow.length; i++) {
            afterWeeks.push(new NtCalendarCell(i + 1, dateNames[i], false));
        }
        lastRow.push(...afterWeeks);
        for (let i = 0; i < DAYS_PER_WEEK * (DAYS_MAX_ROWS - this._weeks.length); i++) {
            afterRows.push(new NtCalendarCell(afterWeeks.length + i + 1, dateNames[afterWeeks.length + i], false));
        }
        this._weeks.push(...afterRows.map(_ => afterRows.splice(0, DAYS_PER_WEEK)).filter(row => !!row));
    }
    _shouldEnableDate(date) {
        return !!date &&
            (!this.minDate || this._dateAdapter.compareDate(date, this.minDate) >= 0) &&
            (!this.maxDate || this._dateAdapter.compareDate(date, this.maxDate) <= 0) &&
            (!this.dateFilter || this.dateFilter(date));
    }
    _getDateInCurrentMonth(date) {
        return date && this._hasSameMonthAndYear(date, this.activeDate) ?
            this._dateAdapter.getDate(date) : null;
    }
    _hasSameMonthAndYear(d1, d2) {
        return !!(d1 && d2 && this._dateAdapter.getMonth(d1) == this._dateAdapter.getMonth(d2) &&
            this._dateAdapter.getYear(d1) == this._dateAdapter.getYear(d2));
    }
    _getCellCompareValue(date) {
        if (date) {
            const year = this._dateAdapter.getYear(date);
            const month = this._dateAdapter.getMonth(date);
            const day = this._dateAdapter.getDate(date);
            return new Date(year, month, day).getTime();
        }
        return null;
    }
    _setRanges(selectedValue) {
        if (selectedValue instanceof DateRange) {
            this._rangeStart = this._getCellCompareValue(selectedValue.start);
            this._rangeEnd = this._getCellCompareValue(selectedValue.end);
            this._isRange = true;
        }
        else {
            this._rangeStart = this._rangeEnd = this._getCellCompareValue(selectedValue);
            this._isRange = false;
        }
        this._comparisonRangeStart = this._getCellCompareValue(this.comparisonStart);
        this._comparisonRangeEnd = this._getCellCompareValue(this.comparisonEnd);
    }
}
NtCalendarMonth.decorators = [
    { type: Component, args: [{
                selector: 'nt-calendar-month',
                template: "<table>\n  <thead>\n    <tr>\n      <th *ngFor=\"let week of _weekdays;\" scope=\"col\">\n        {{week.narrow}}\n      </th>\n    </tr>\n  </thead>\n  <tbody nt-calendar-body\n    [label]=\"_monthLabel\"\n    [rows]=\"_weeks\"\n    [todayValue]=\"_todayDate!\"\n    [startValue]=\"_rangeStart!\"\n    [endValue]=\"_rangeEnd!\"\n    [comparisonStart]=\"_comparisonRangeStart\"\n    [comparisonEnd]=\"_comparisonRangeEnd\"\n    [previewStart]=\"_previewStart\"\n    [previewEnd]=\"_previewEnd\"\n    [isRange]=\"_isRange\"\n    [activeCell]=\"_dateAdapter.getDate(activeDate) - 1\"\n    (selectedValueChange)=\"_dateSelected($event)\"\n    (previewChange)=\"_previewChanged($event)\">\n  </tbody>\n</table>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-calendar-month'
                }
            },] }
];
NtCalendarMonth.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_RANGE_SELECTION_STRATEGY,] }, { type: Optional }] }
];
NtCalendarMonth.propDecorators = {
    activeDate: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    dateClass: [{ type: Input }],
    comparisonStart: [{ type: Input }],
    comparisonEnd: [{ type: Input }],
    selectedChange: [{ type: Output }],
    _userSelection: [{ type: Output }],
    activeDateChange: [{ type: Output }],
    _calendarBody: [{ type: ViewChild, args: [NtCalendarBody,] }]
};

class NtCalendarMultiYear {
    constructor(_changeDetectorRef, _dateAdapter) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._rerenderSubscription = Subscription.EMPTY;
        this.selectedChange = new EventEmitter();
        this.yearSelected = new EventEmitter();
        this.activeDateChange = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        this._activeDate = this._dateAdapter.today();
    }
    get activeDate() { return this._activeDate; }
    set activeDate(value) {
        let oldActiveDate = this._activeDate;
        const validDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value)) || this._dateAdapter.today();
        this._activeDate = this._dateAdapter.clampDate(validDate, this.minDate, this.maxDate);
        if (!isSameMultiYearView(this._dateAdapter, oldActiveDate, this._activeDate, this.minDate, this.maxDate)) {
            this._init();
        }
    }
    get selected() { return this._selected; }
    set selected(value) {
        if (value instanceof DateRange) {
            this._selected = value;
        }
        else {
            this._selected = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
        }
        this._setSelectedYear(value);
    }
    get minDate() { return this._minDate; }
    set minDate(value) {
        this._minDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get maxDate() { return this._maxDate; }
    set maxDate(value) {
        this._maxDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    ngAfterContentInit() {
        this._rerenderSubscription = this._dateAdapter.localeChanges
            .pipe(startWith(null))
            .subscribe(() => this._init());
    }
    ngOnDestroy() {
        this._rerenderSubscription.unsubscribe();
    }
    _init() {
        this._todayYear = this._dateAdapter.getYear(this._dateAdapter.today());
        let activeYear = this._dateAdapter.getYear(this._activeDate);
        let activeOffset = activeYear % yearsPerPage;
        this._years = [];
        for (let i = 0, row = []; i < yearsPerPage; i++) {
            row.push(activeYear - activeOffset + i);
            if (row.length === yearsPerRow) {
                this._years.push(row.map(year => this._createCellForYear(year)));
                row = [];
            }
        }
        this._changeDetectorRef.markForCheck();
    }
    _yearSelected(event) {
        const year = event.value;
        this.yearSelected.emit(this._dateAdapter.createDate(year, 0, 1));
        let month = this._dateAdapter.getMonth(this.activeDate);
        let daysInMonth = this._dateAdapter.getNumDaysInMonth(this._dateAdapter.createDate(year, month, 1));
        this.selectedChange.emit(this._dateAdapter.createDate(year, month, Math.min(this._dateAdapter.getDate(this.activeDate), daysInMonth)));
    }
    _getActiveCell() {
        return getActiveOffset(this._dateAdapter, this.activeDate, this.minDate, this.maxDate);
    }
    _focusActiveCell() {
        this._calendarBody._focusActiveCell();
    }
    _createCellForYear(year) {
        const date = this._dateAdapter.createDate(year, 0, 1);
        const yearName = this._dateAdapter.getYearName(date);
        const cellClasses = this.dateClass ? this.dateClass(date, 'multi-year') : undefined;
        return new NtCalendarCell(year, yearName, this._shouldEnableYear(year), cellClasses);
    }
    _shouldEnableYear(year) {
        if (year === undefined || year === null ||
            (this.maxDate && year > this._dateAdapter.getYear(this.maxDate)) ||
            (this.minDate && year < this._dateAdapter.getYear(this.minDate))) {
            return false;
        }
        if (!this.dateFilter) {
            return true;
        }
        const firstOfYear = this._dateAdapter.createDate(year, 0, 1);
        for (let date = firstOfYear; this._dateAdapter.getYear(date) === year; date = this._dateAdapter.addCalendarDays(date, 1)) {
            if (this.dateFilter(date)) {
                return true;
            }
        }
        return false;
    }
    _setSelectedYear(value) {
        this._selectedYear = null;
        if (value instanceof DateRange) {
            const displayValue = value.start || value.end;
            if (displayValue) {
                this._selectedYear = this._dateAdapter.getYear(displayValue);
            }
        }
        else if (value) {
            this._selectedYear = this._dateAdapter.getYear(value);
        }
    }
}
NtCalendarMultiYear.decorators = [
    { type: Component, args: [{
                selector: 'nt-calendar-multi-year',
                template: "<table>\n  <tbody nt-calendar-body\n    [rows]=\"_years\"\n    [todayValue]=\"_todayYear\"\n    [startValue]=\"_selectedYear!\"\n    [endValue]=\"_selectedYear!\"\n    [numCols]=\"3\"\n    [cellAspectRatio]=\"3 / 7\"\n    [activeCell]=\"_getActiveCell()\"\n    (selectedValueChange)=\"_yearSelected($event)\">\n  </tbody>\n</table>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-calendar-multi-year'
                }
            },] }
];
NtCalendarMultiYear.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] }
];
NtCalendarMultiYear.propDecorators = {
    activeDate: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    dateClass: [{ type: Input }],
    selectedChange: [{ type: Output }],
    yearSelected: [{ type: Output }],
    activeDateChange: [{ type: Output }],
    _calendarBody: [{ type: ViewChild, args: [NtCalendarBody,] }]
};

class NtCalendarYear {
    constructor(_changeDetectorRef, _dateAdapter, _dateFormats) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        this._rerenderSubscription = Subscription.EMPTY;
        this.selectedChange = new EventEmitter();
        this.monthSelected = new EventEmitter();
        this.activeDateChange = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        if (!this._dateFormats) {
            throw createMissingDateImplError('NT_DATE_FORMATS');
        }
        this._activeDate = this._dateAdapter.today();
    }
    get activeDate() { return this._activeDate; }
    set activeDate(value) {
        let oldActiveDate = this._activeDate;
        const validDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value)) || this._dateAdapter.today();
        this._activeDate = this._dateAdapter.clampDate(validDate, this.minDate, this.maxDate);
        if (this._dateAdapter.getYear(oldActiveDate) !== this._dateAdapter.getYear(this._activeDate)) {
            this._init();
        }
    }
    get selected() { return this._selected; }
    set selected(value) {
        if (value instanceof DateRange) {
            this._selected = value;
        }
        else {
            this._selected = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
        }
    }
    get minDate() { return this._minDate; }
    set minDate(value) {
        this._minDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get maxDate() { return this._maxDate; }
    set maxDate(value) {
        this._maxDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    ngAfterContentInit() {
        this._rerenderSubscription = this._dateAdapter.localeChanges
            .pipe(startWith(null))
            .subscribe(() => this._init());
    }
    ngOnDestroy() {
        this._rerenderSubscription.unsubscribe();
    }
    _init() {
        this._setSelectedMonth(this.selected);
        this._todayMonth = this._getMonthInCurrentYear(this._dateAdapter.today());
        this._yearLabel = this._dateAdapter.getYearName(this.activeDate);
        const monthNames = this._dateAdapter.getMonthNames('short');
        this._months = [[0, 1, 2], [3, 4, 5], [6, 7, 8], [9, 10, 11]].map(row => row.map(month => this._createCellForMonth(month, monthNames[month])));
        this._changeDetectorRef.markForCheck();
    }
    _monthSelected(event) {
        const month = event.value;
        const normalizedDate = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), month, 1);
        this.monthSelected.emit(normalizedDate);
        const daysInMonth = this._dateAdapter.getNumDaysInMonth(normalizedDate);
        this.selectedChange.emit(this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), month, Math.min(this._dateAdapter.getDate(this.activeDate), daysInMonth)));
    }
    _focusActiveCell() {
        this._calendarBody._focusActiveCell();
    }
    _getMonthInCurrentYear(date) {
        return date && this._dateAdapter.getYear(date) == this._dateAdapter.getYear(this.activeDate) ?
            this._dateAdapter.getMonth(date) : null;
    }
    _createCellForMonth(month, monthName) {
        const date = this._dateAdapter.createDate(this._dateAdapter.getYear(this.activeDate), month, 1);
        const cellClasses = this.dateClass ? this.dateClass(date, 'year') : undefined;
        return new NtCalendarCell(month, monthName.toLocaleUpperCase(), this._shouldEnableMonth(month), cellClasses);
    }
    _shouldEnableMonth(month) {
        const activeYear = this._dateAdapter.getYear(this.activeDate);
        if (month === undefined || month === null ||
            this._isYearAndMonthAfterMaxDate(activeYear, month) ||
            this._isYearAndMonthBeforeMinDate(activeYear, month)) {
            return false;
        }
        if (!this.dateFilter) {
            return true;
        }
        const firstOfMonth = this._dateAdapter.createDate(activeYear, month, 1);
        for (let date = firstOfMonth; this._dateAdapter.getMonth(date) == month; date = this._dateAdapter.addCalendarDays(date, 1)) {
            if (this.dateFilter(date)) {
                return true;
            }
        }
        return false;
    }
    _isYearAndMonthAfterMaxDate(year, month) {
        if (this.maxDate) {
            const maxYear = this._dateAdapter.getYear(this.maxDate);
            const maxMonth = this._dateAdapter.getMonth(this.maxDate);
            return year > maxYear || (year === maxYear && month > maxMonth);
        }
        return false;
    }
    _isYearAndMonthBeforeMinDate(year, month) {
        if (this.minDate) {
            const minYear = this._dateAdapter.getYear(this.minDate);
            const minMonth = this._dateAdapter.getMonth(this.minDate);
            return year < minYear || (year === minYear && month < minMonth);
        }
        return false;
    }
    _setSelectedMonth(value) {
        if (value instanceof DateRange) {
            this._selectedMonth = this._getMonthInCurrentYear(value.start) ||
                this._getMonthInCurrentYear(value.end);
        }
        else {
            this._selectedMonth = this._getMonthInCurrentYear(value);
        }
    }
}
NtCalendarYear.decorators = [
    { type: Component, args: [{
                selector: 'nt-calendar-year',
                template: "<table>\n  <tbody nt-calendar-body\n    [label]=\"_yearLabel\"\n    [rows]=\"_months\"\n    [todayValue]=\"_todayMonth!\"\n    [startValue]=\"_selectedMonth!\"\n    [endValue]=\"_selectedMonth!\"\n    [numCols]=\"3\"\n    [cellAspectRatio]=\"3 / 7\"\n    [activeCell]=\"_dateAdapter.getMonth(activeDate)\"\n    (selectedValueChange)=\"_monthSelected($event)\">\n  </tbody>\n</table>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-calendar-year'
                }
            },] }
];
NtCalendarYear.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] }
];
NtCalendarYear.propDecorators = {
    activeDate: [{ type: Input }],
    selected: [{ type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    dateClass: [{ type: Input }],
    selectedChange: [{ type: Output }],
    monthSelected: [{ type: Output }],
    activeDateChange: [{ type: Output }],
    _calendarBody: [{ type: ViewChild, args: [NtCalendarBody,] }]
};

class NtDatePickerCalendar {
    constructor(_changeDetectorRef, _dateAdapter, _dateFormats) {
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        this._moveFocusOnNextTick = false;
        this.startView = 'month';
        this.stateChanges = new Subject();
        this.selectedChange = new EventEmitter();
        this.yearSelected = new EventEmitter();
        this.monthSelected = new EventEmitter();
        this.viewChanged = new EventEmitter(true);
        this._userSelection = new EventEmitter();
        if (!this._dateAdapter) {
            throw createMissingDateImplError('DateAdapter');
        }
        if (!this._dateFormats) {
            throw createMissingDateImplError('NT_DATE_FORMATS');
        }
    }
    get startAt() { return this._startAt; }
    set startAt(value) {
        this._startAt = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get selected() { return this._selected; }
    set selected(value) {
        if (value instanceof DateRange) {
            this._selected = value;
        }
        else {
            this._selected = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
        }
    }
    get minDate() { return this._minDate; }
    set minDate(value) {
        this._minDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get maxDate() { return this._maxDate; }
    set maxDate(value) {
        this._maxDate = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get currentView() { return this._currentView; }
    set currentView(value) {
        const viewChangedResult = this._currentView !== value ? value : null;
        this._currentView = value;
        this._moveFocusOnNextTick = true;
        this._changeDetectorRef.markForCheck();
        if (viewChangedResult) {
            this.viewChanged.emit(viewChangedResult);
        }
    }
    get activeDate() { return this._clampedActiveDate; }
    set activeDate(value) {
        this._clampedActiveDate = this._dateAdapter.clampDate(value, this.minDate, this.maxDate);
        this.stateChanges.next();
        this._changeDetectorRef.markForCheck();
    }
    ngAfterContentInit() {
        this.activeDate = this.startAt || this._dateAdapter.today();
        this._currentView = this.startView;
    }
    ngOnChanges(changes) {
        const change = changes['minDate'] || changes['maxDate'] || changes['dateFilter'];
        if (change && !change.firstChange) {
            const view = this._getCurrentViewComponent();
            if (view) {
                this._changeDetectorRef.detectChanges();
                view._init();
            }
        }
        this.stateChanges.next();
    }
    ngAfterViewChecked() {
        if (this._moveFocusOnNextTick) {
            this._moveFocusOnNextTick = false;
            this.focusActiveCell();
        }
    }
    ngOnDestroy() {
        this.stateChanges.complete();
    }
    focusActiveCell() {
        this._getCurrentViewComponent()._focusActiveCell(false);
    }
    updateTodaysDate() {
        const currentView = this.currentView;
        let view;
        if (currentView === 'month') {
            view = this.monthView;
        }
        else if (currentView === 'year') {
            view = this.yearView;
        }
        else {
            view = this.multiYearView;
        }
        view._init();
    }
    _dateSelected(event) {
        const date = event.value;
        if (this.selected instanceof DateRange ||
            (date && !this._dateAdapter.sameDate(date, this.selected))) {
            this.selectedChange.emit(date);
        }
        this._userSelection.emit(event);
    }
    _yearSelectedInMultiYearView(normalizedYear) {
        this.yearSelected.emit(normalizedYear);
    }
    _monthSelectedInYearView(normalizedMonth) {
        this.monthSelected.emit(normalizedMonth);
    }
    _goToDateInView(date, view) {
        this.activeDate = date;
        this.currentView = view;
    }
    _getCurrentViewComponent() {
        return this.monthView || this.yearView || this.multiYearView;
    }
}
NtDatePickerCalendar.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker-calendar',
                template: "<nt-calendar-header></nt-calendar-header>\n\n<ng-container [ngSwitch]=\"currentView\">\n  <nt-calendar-month\n    *ngSwitchCase=\"'month'\"\n    [(activeDate)]=\"activeDate\"\n    [selected]=\"selected\"\n    [dateFilter]=\"dateFilter\"\n    [maxDate]=\"maxDate\"\n    [minDate]=\"minDate\"\n    [dateClass]=\"dateClass\"\n    [comparisonStart]=\"comparisonStart\"\n    [comparisonEnd]=\"comparisonEnd\"\n    (_userSelection)=\"_dateSelected($event)\">\n  </nt-calendar-month>\n  <nt-calendar-year\n    *ngSwitchCase=\"'year'\"\n    [(activeDate)]=\"activeDate\"\n    [selected]=\"selected\"\n    [dateFilter]=\"dateFilter\"\n    [maxDate]=\"maxDate\"\n    [minDate]=\"minDate\"\n    [dateClass]=\"dateClass\"\n    (monthSelected)=\"_monthSelectedInYearView($event)\"\n    (selectedChange)=\"_goToDateInView($event, 'month')\">\n  </nt-calendar-year>\n  <nt-calendar-multi-year\n    *ngSwitchCase=\"'multi-year'\"\n    [(activeDate)]=\"activeDate\"\n    [selected]=\"selected\"\n    [dateFilter]=\"dateFilter\"\n    [maxDate]=\"maxDate\"\n    [minDate]=\"minDate\"\n    [dateClass]=\"dateClass\"\n    (yearSelected)=\"_yearSelectedInMultiYearView($event)\"\n    (selectedChange)=\"_goToDateInView($event, 'year')\">\n  </nt-calendar-multi-year>\n</ng-container>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-datepicker-calendar'
                }
            },] }
];
NtDatePickerCalendar.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] }
];
NtDatePickerCalendar.propDecorators = {
    startView: [{ type: Input }],
    startAt: [{ type: Input }],
    selected: [{ type: Input }, { type: Input }],
    minDate: [{ type: Input }],
    maxDate: [{ type: Input }],
    dateFilter: [{ type: Input }],
    dateClass: [{ type: Input }],
    comparisonStart: [{ type: Input }],
    comparisonEnd: [{ type: Input }],
    selectedChange: [{ type: Output }],
    yearSelected: [{ type: Output }],
    monthSelected: [{ type: Output }],
    viewChanged: [{ type: Output }],
    _userSelection: [{ type: Output }],
    monthView: [{ type: ViewChild, args: [NtCalendarMonth,] }],
    yearView: [{ type: ViewChild, args: [NtCalendarYear,] }],
    multiYearView: [{ type: ViewChild, args: [NtCalendarMultiYear,] }]
};

class NtCalendarHeader {
    constructor(calendar, _dateAdapter, _dateFormats, changeDetectorRef) {
        this.calendar = calendar;
        this._dateAdapter = _dateAdapter;
        this._dateFormats = _dateFormats;
        this.calendar.stateChanges.subscribe(() => changeDetectorRef.markForCheck());
    }
    get _monthLabel() {
        return this._dateAdapter.format(this.calendar.activeDate, this._dateFormats.display.monthLabel).toLocaleUpperCase();
    }
    get _yearLabel() {
        return this._dateAdapter.getYearName(this.calendar.activeDate);
    }
    get _multiYearLabel() {
        const activeYear = this._dateAdapter.getYear(this.calendar.activeDate);
        const firstYearInView = this._dateAdapter.getYearName(this._dateAdapter.createDate(activeYear - activeYear % yearsPerPage, 0, 1), 'en-US');
        const lastYearInView = this._dateAdapter.getYearName(this._dateAdapter.createDate(activeYear + yearsPerPage - 1 - activeYear % yearsPerPage, 0, 1), 'en-US');
        return `${firstYearInView} \u2013 ${lastYearInView}`;
    }
    currentPeriodClicked() {
        this.calendar.currentView = this.calendar.currentView == 'month' ? 'multi-year' : 'month';
    }
    prevMonth() {
        this.calendar.activeDate = this._dateAdapter.addCalendarMonths(this.calendar.activeDate, -1);
    }
    nextMonth() {
        this.calendar.activeDate = this._dateAdapter.addCalendarMonths(this.calendar.activeDate, 1);
    }
    prevYear() {
        this.calendar.activeDate = this._dateAdapter.addCalendarYears(this.calendar.activeDate, -1);
    }
    nextYear() {
        this.calendar.activeDate = this._dateAdapter.addCalendarYears(this.calendar.activeDate, 1);
    }
    prevYearArray() {
        this.calendar.activeDate = this._dateAdapter.addCalendarYears(this.calendar.activeDate, -yearsPerPage);
    }
    nextYearArray() {
        this.calendar.activeDate = this._dateAdapter.addCalendarYears(this.calendar.activeDate, yearsPerPage);
    }
    previousEnabled(viewType = 'month') {
        if (!this.calendar.minDate) {
            return true;
        }
        return !this.calendar.minDate ||
            !this._isSameView(this.calendar.activeDate, this.calendar.minDate, viewType);
    }
    nextEnabled(viewType = 'month') {
        return !this.calendar.maxDate ||
            !this._isSameView(this.calendar.activeDate, this.calendar.maxDate, viewType);
    }
    changeCurrentView(viewType) {
        this.calendar.currentView = viewType;
    }
    _isSameView(date1, date2, viewType = 'month') {
        if (viewType == 'month') {
            return this._dateAdapter.getYear(date1) == this._dateAdapter.getYear(date2) &&
                this._dateAdapter.getMonth(date1) == this._dateAdapter.getMonth(date2);
        }
        if (viewType == 'year') {
            return this._dateAdapter.getYear(date1) == this._dateAdapter.getYear(date2);
        }
        return isSameMultiYearView(this._dateAdapter, date1, date2, this.calendar.minDate, this.calendar.maxDate);
    }
}
NtCalendarHeader.decorators = [
    { type: Component, args: [{
                selector: 'nt-calendar-header',
                template: "<ng-container [ngSwitch]=\"calendar.currentView\">\n  <table>\n    <thead>\n      <tr *ngSwitchCase=\"'month'\"><ng-container *ngTemplateOutlet=\"monthControl\"></ng-container></tr>\n      <tr *ngSwitchCase=\"'year'\"><ng-container *ngTemplateOutlet=\"yearControl\"></ng-container></tr>\n      <tr *ngSwitchCase=\"'multi-year'\"><ng-container *ngTemplateOutlet=\"multiYearControl\"></ng-container></tr>\n    </thead>\n  </table>\n\n  <ng-template #monthControl>\n    <th (click)=\"prevYear()\"\n      class=\"nt-calendar-cell\"\n      [class.nt-calendar-cell-disabled]=\"!previousEnabled('year')\">\n      <span>&laquo;</span>\n    </th>\n    <th (click)=\"prevMonth()\"\n      class=\"nt-calendar-cell\"\n      [class.nt-calendar-cell-disabled]=\"!previousEnabled()\">\n      <span>&lsaquo;</span>\n    </th>\n    <th colspan=\"3\" class=\"nt-calendar-cell\">\n      <span (click)=\" calendar.currentView = 'multi-year' \">{{ _yearLabel }}</span>\n      <span (click)=\" calendar.currentView = 'year' \">{{ _monthLabel }}</span>\n    </th>\n    <th (click)=\"nextMonth()\"\n      class=\"nt-calendar-cell\"\n      [class.nt-calendar-cell-disabled]=\"!nextEnabled()\">\n      <span>&rsaquo;</span>\n    </th>\n    <th (click)=\"nextYear()\"\n      class=\"nt-calendar-cell\"\n      [class.nt-calendar-cell-disabled]=\"!nextEnabled('year')\">\n      <span>&raquo;</span>\n    </th>\n  </ng-template>\n\n  <ng-template #yearControl>\n    <th (click)=\"prevYear()\"\n      class=\"nt-calendar-cell\"\n      [class.nt-calendar-cell-disabled]=\"!previousEnabled('year')\">\n      <span>&laquo;</span>\n    </th>\n    <th>&nbsp;</th>\n    <th class=\"nt-calendar-cell\" colspan=\"3\">\n      <span (click)=\" calendar.currentView = 'multi-year' \">\n        {{ _yearLabel }}\n      </span>\n    </th>\n    <th>&nbsp;</th>\n    <th (click)=\"nextYear()\"\n      class=\"nt-calendar-cell\"\n      [class.nt-calendar-cell-disabled]=\"!nextEnabled('year')\">\n      <span>&raquo;</span>\n    </th>\n  </ng-template>\n\n  <ng-template #multiYearControl>\n    <th (click)=\"prevYearArray()\" class=\"nt-calendar-cell\">\n      <span>&laquo;</span>\n    </th>\n    <th class=\"nt-calendar-cell\">&nbsp;</th>\n    <th class=\"nt-calendar-cell\" colspan=\"3\">{{_multiYearLabel}}</th>\n    <th>&nbsp;</th>\n    <th (click)=\"nextYearArray()\" class=\"nt-calendar-cell\">\n      <span>&raquo;</span>\n    </th>\n  </ng-template>\n\n</ng-container>\n",
                exportAs: 'ntCalendarHeader',
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-calendar-header'
                }
            },] }
];
NtCalendarHeader.ctorParameters = () => [
    { type: NtDatePickerCalendar, decorators: [{ type: Inject, args: [forwardRef(() => NtDatePickerCalendar),] }] },
    { type: DateAdapter, decorators: [{ type: Optional }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_FORMATS,] }] },
    { type: ChangeDetectorRef }
];

class NtDatePickerInputBase {
    constructor(_dateAdapter) {
        this._dateAdapter = _dateAdapter;
        this._disabled = false;
        this._required = false;
        this._lastValueValid = false;
        this._onChange = () => { };
        this._onTouched = () => { };
        this._validatorOnChange = () => { };
        this.stateChanges = new Subject();
        this._valueChangesSubscription = Subscription.EMPTY;
        this._localeSubscription = Subscription.EMPTY;
        this._parseValidator = () => {
            return this._lastValueValid ?
                null : { 'ntDatePickerParse': { 'text': this.value } };
        };
        this._filterValidator = (control) => {
            const controlValue = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(control.value));
            const dateFilter = this._getDateFilter();
            return !dateFilter || !controlValue || dateFilter(controlValue)
                ? null
                : { ntDatePickerFilter: true };
        };
        this._minValidator = (control) => {
            const controlValue = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(control.value));
            const min = this._getMinDate();
            return !min ||
                !controlValue ||
                this._dateAdapter.compareDate(min, controlValue) <= 0
                ? null
                : { ntDatePickerMin: { min: min, actual: controlValue } };
        };
        this._maxValidator = (control) => {
            const controlValue = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(control.value));
            const max = this._getMaxDate();
            return !max ||
                !controlValue ||
                this._dateAdapter.compareDate(max, controlValue) >= 0
                ? null
                : { ntDatePickerMax: { max: max, actual: controlValue } };
        };
        this._localeSubscription = _dateAdapter.localeChanges.subscribe(() => {
            this.value = this.value;
        });
    }
    get value() {
        return this._model
            ? this._getValueFromModel(this._model.selection)
            : this._value;
    }
    set value(value) {
        value = this._dateAdapter.deserialize(value);
        this._lastValueValid = this._isValidValue(value);
        value = this._dateAdapter.getValidDateOrNull(value);
        const oldDate = this.value;
        this._assignValue(value);
        if (!this._dateAdapter.sameDate(oldDate, value)) {
            this._valueChange(value);
        }
    }
    get disabled() { return this._disabled; }
    set disabled(value) {
        this._disabled = coerceBooleanProperty(value);
    }
    get required() {
        var _a, _b;
        if ((_b = (_a = this.ngControl) === null || _a === void 0 ? void 0 : _a.control) === null || _b === void 0 ? void 0 : _b.validator) {
            const control = new FormControl();
            const validateResult = this.ngControl.control.validator(control);
            return Object.prototype.hasOwnProperty.call(validateResult, 'required');
        }
        return this._required;
    }
    set required(value) {
        this._required = coerceBooleanProperty(value);
    }
    ngOnInit() { }
    ngOnDestroy() {
        this._valueChangesSubscription.unsubscribe();
        this._localeSubscription.unsubscribe();
        this.stateChanges.complete();
    }
    writeValue(value) {
        this.value = value;
    }
    registerOnChange(fn) {
        this._onChange = fn;
    }
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
    setDisabledState(isDisabled) {
        this.disabled = isDisabled;
    }
    registerOnValidatorChange(fn) {
        this._validatorOnChange = fn;
    }
    validate(c) {
        return this._validator ? this._validator(c) : null;
    }
    _valueChange(value) {
        this._onChange(value);
    }
    _registerModel(model) {
        this._model = model;
        this._valueChangesSubscription.unsubscribe();
        if (this._value) {
            this._assignValue(this._value);
        }
        this._valueChangesSubscription = this._model.selectionChanged.subscribe((event) => {
            if (event.source !== this) {
                const value = this._getValueFromModel(event.selection);
                this._lastValueValid = this._isValidValue(value);
                this._valueChange(value);
                this._onTouched();
            }
        });
    }
    _assignValue(value) {
        if (this._model) {
            this._assignValueToModel(value);
            this._value = null;
        }
        else {
            this._value = value;
        }
    }
    _isValidValue(value) {
        return !value || this._dateAdapter.isValid(value);
    }
    _getValidators() {
        return [
            this._parseValidator,
            this._minValidator,
            this._maxValidator,
            this._filterValidator,
        ];
    }
}
NtDatePickerInputBase.decorators = [
    { type: Directive }
];
NtDatePickerInputBase.ctorParameters = () => [
    { type: DateAdapter }
];
NtDatePickerInputBase.propDecorators = {
    disabled: [{ type: Input }],
    required: [{ type: Input }]
};

const NT_DATE_RANGE_PICKER_PARENT = new InjectionToken('nt-date-range-parent');
class NtDateRangePart extends NtDatePickerInputBase {
    constructor(dateAdapter, _rangePicker, _injector) {
        super(dateAdapter);
        this._rangePicker = _rangePicker;
        this._injector = _injector;
    }
    ngOnInit() {
        const ngControl = this._injector.get(NgControl, null, InjectFlags.Self);
        if (ngControl) {
            this.ngControl = ngControl;
        }
    }
    _getMinDate() {
        return this._rangePicker.min;
    }
    _getMaxDate() {
        return this._rangePicker.max;
    }
    _getDateFilter() {
        return this._rangePicker.dateFilter;
    }
}
NtDateRangePart.decorators = [
    { type: Directive }
];
NtDateRangePart.ctorParameters = () => [
    { type: DateAdapter },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_RANGE_PICKER_PARENT,] }] },
    { type: Injector }
];
class NtDateRangeStart extends NtDateRangePart {
    constructor() {
        super(...arguments);
        this._startValidator = (control) => {
            const start = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(control.value));
            const end = this._model ? this._model.selection.end : null;
            return (!start || !end ||
                this._dateAdapter.compareDate(start, end) <= 0) ?
                null : { 'ntDateRangeStartInvalid': { 'end': end, 'actual': start } };
        };
        this._validator = Validators.compose([...super._getValidators(), this._startValidator]);
    }
    _assignValueToModel(value) {
        if (this._model) {
            const range = new DateRange(value, this._model.selection.end);
            this._model.updateSelection(range, this);
            this._valueChange(value);
        }
    }
    _getValueFromModel(modelValue) {
        return modelValue.start;
    }
}
NtDateRangeStart.decorators = [
    { type: Directive, args: [{
                selector: 'nt-date-range-start',
                providers: [
                    { provide: NG_VALUE_ACCESSOR, useExisting: NtDateRangeStart, multi: true },
                    { provide: NG_VALIDATORS, useExisting: NtDateRangeStart, multi: true }
                ],
            },] }
];
class NtDateRangeEnd extends NtDateRangePart {
    constructor() {
        super(...arguments);
        this._endValidator = (control) => {
            const end = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(control.value));
            const start = this._model ? this._model.selection.start : null;
            return (!end || !start ||
                this._dateAdapter.compareDate(end, start) >= 0) ?
                null : { 'ntDateRangeEndInvalid': { 'start': start, 'actual': end } };
        };
        this._validator = Validators.compose([...super._getValidators(), this._endValidator]);
    }
    _assignValueToModel(value) {
        if (this._model) {
            const range = new DateRange(this._model.selection.start, value);
            this._model.updateSelection(range, this);
            this._valueChange(value);
        }
    }
    _getValueFromModel(modelValue) {
        return modelValue.end;
    }
}
NtDateRangeEnd.decorators = [
    { type: Directive, args: [{
                selector: 'nt-date-range-end',
                providers: [
                    { provide: NG_VALUE_ACCESSOR, useExisting: NtDateRangeEnd, multi: true },
                    { provide: NG_VALIDATORS, useExisting: NtDateRangeEnd, multi: true }
                ],
            },] }
];

const NT_DATE_PICKER_CONTROL = new InjectionToken('nt-datepicker-control');

class NtDatePickerContent {
    constructor(elementRef, _changeDetectorRef, _model, _dateAdapter, _rangeSelectionStrategy, datepicker) {
        this.elementRef = elementRef;
        this._changeDetectorRef = _changeDetectorRef;
        this._model = _model;
        this._dateAdapter = _dateAdapter;
        this._rangeSelectionStrategy = _rangeSelectionStrategy;
        this.datepicker = datepicker;
        this._subscriptions = new Subscription();
    }
    ngAfterViewInit() {
        this._subscriptions.add(this.datepicker._stateChanges.subscribe(() => {
            this._changeDetectorRef.markForCheck();
        }));
        this._calendar.focusActiveCell();
    }
    ngOnDestroy() {
        this._subscriptions.unsubscribe();
    }
    _handleUserSelection(event) {
        const selection = this._model.selection;
        const value = event.value;
        const isRange = selection instanceof DateRange;
        if (isRange && this._rangeSelectionStrategy) {
            const newSelection = this._rangeSelectionStrategy.selectionFinished(value, selection, event.event);
            this._model.updateSelection(newSelection, this);
        }
        else if (value && (isRange ||
            !this._dateAdapter.sameDate(value, selection))) {
            this._model.add(value);
        }
        if (!this._model || this._model.isComplete()) {
            this.datepicker.overlay.close();
        }
    }
    _getSelected() {
        return this._model.selection;
    }
}
NtDatePickerContent.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker-content',
                template: "<div cdkTrapFocus>\n  <nt-datepicker-calendar\n    [id]=\"datepicker.id\"\n    [startAt]=\"datepicker.startAt\"\n    [minDate]=\"datepicker.min\"\n    [maxDate]=\"datepicker.max\"\n    [dateFilter]=\"datepicker.dateFilter\"\n    [selected]=\"_getSelected()\"\n    [dateClass]=\"datepicker.dateClass\"\n    [comparisonStart]=\"comparisonStart\"\n    [comparisonEnd]=\"comparisonEnd\"\n    (_userSelection)=\"_handleUserSelection($event)\">\n  </nt-datepicker-calendar>\n</div>\n",
                host: {
                    'class': 'nt-datepicker-content',
                },
                exportAs: 'ntDatepickerContent',
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush
            },] }
];
NtDatePickerContent.ctorParameters = () => [
    { type: ElementRef },
    { type: ChangeDetectorRef },
    { type: NtDateSelectionModel },
    { type: DateAdapter },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATE_RANGE_SELECTION_STRATEGY,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_PICKER_CONTROL,] }] }
];
NtDatePickerContent.propDecorators = {
    _calendar: [{ type: ViewChild, args: [NtDatePickerCalendar,] }]
};

const DEFAULT_DATEPICKER_ICONS = {
    caret: 'far fa-calendar-alt',
    clear: 'fa fa-times'
};
const NT_DATEPICKER_ICONS = new InjectionToken('nt-datepicker-icons');

class NtDatePickerBase {
    constructor(elementRef, tabIndex, icons, _dateFormats, _changeDetectorRef, _dateAdapter, _model) {
        this.icons = icons;
        this._dateFormats = _dateFormats;
        this._changeDetectorRef = _changeDetectorRef;
        this._dateAdapter = _dateAdapter;
        this._model = _model;
        this._overlayToggle = new Subject();
        this._positionPairs = [BOTTOM_LEFT, TOP_LEFT];
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this._stateChanges = new Subject();
        this.origin = new CdkOverlayOrigin(elementRef);
        this.icons = Object.assign(Object.assign({}, DEFAULT_DATEPICKER_ICONS), icons);
        this.tabIndex = parseInt(tabIndex) || 0;
        this._overlayToggle.pipe(debounceTime(10)).subscribe(open => open
            ? this.overlay.open()
            : this.overlay.toggle());
        this._model.selectionChanged.subscribe(value => {
            this._formatValue(value.selection);
        });
    }
    get startAt() { return this._startAt || this._getStartValue(); }
    set startAt(value) {
        this._startAt = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get min() { return this._min; }
    set min(value) {
        this._min = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    get max() { return this._max; }
    set max(value) {
        this._max = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value));
    }
    ngAfterViewInit() {
        this._changeDetectorRef.markForCheck();
    }
    ngOnChanges(changes) {
        const change = changes.value || changes.startAt || changes.minDate || changes.maxDate;
        if (change && !change.firstChange) {
            this._changeDetectorRef.markForCheck();
        }
        this._stateChanges.next(undefined);
    }
    ngOnDestroy() {
        this._stateChanges.complete();
    }
    focus() {
        this.overlay.open();
    }
    select(date) {
        this._model.add(date);
    }
    _onAfterOpen(event) {
        this.afterOpen.next(event);
    }
    _onAfterClosed(event) {
        this.afterClosed.next(event);
        this._portal.detach();
    }
    _onBeforeOpen(event) {
        this._portal = new ComponentPortal(NtDatePickerContent);
        this.beforeOpen.next(event);
    }
    _onBeforeClosed(event) {
        this.beforeClosed.next(event);
    }
    _onPositionChange(change) {
        this.positionChange.next(change);
    }
    _onClick(event) {
        if (this._isActivated()) {
            this.overlay.toggle();
        }
        event.stopPropagation();
    }
    _onClear(event) {
        if (this._isActivated()) {
            this._model.updateSelection(this._getDefaultModelValue(), this);
        }
        event.stopPropagation();
    }
}
NtDatePickerBase.decorators = [
    { type: Directive }
];
NtDatePickerBase.ctorParameters = () => [
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATEPICKER_ICONS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_FORMATS,] }] },
    { type: ChangeDetectorRef },
    { type: DateAdapter },
    { type: NtDateSelectionModel }
];
NtDatePickerBase.propDecorators = {
    startAt: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    dateClass: [{ type: Input }],
    dateFilter: [{ type: Input }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }],
    _portalOutlet: [{ type: ViewChild, args: [CdkPortalOutlet, { static: true },] }]
};

let datepickerUid = 0;
class NtDateRangePicker extends NtDatePickerBase {
    constructor(elementRef, tabIndex, icons, dateFormats, changeDetectorRef, dateAdapter, model, control) {
        super(elementRef, tabIndex, icons, dateFormats, changeDetectorRef, dateAdapter, model);
        this.id = `nt-date-range-picker-${datepickerUid++}`;
        this._displayValue = '';
        this._placeholder = '';
        this._disabled = false;
        this._required = false;
        this._readonly = false;
        this.ngControl = control;
    }
    get placeholder() { return this._placeholder; }
    set placeholder(value) {
        this._placeholder = value;
    }
    get disabled() { return this._disabled; }
    set disabled(value) {
        this._disabled = coerceBooleanProperty(value);
    }
    get required() { return this._startDate.required || this._endDate.required || this._required; }
    set required(value) {
        this._required = coerceBooleanProperty(value);
    }
    get readonly() { return this._readonly; }
    set readonly(value) {
        this._readonly = coerceBooleanProperty(value);
    }
    ngAfterContentInit() {
        this._startDate._registerModel(this._model);
        this._endDate._registerModel(this._model);
    }
    _isActivated() {
        return !this.disabled;
    }
    _getStartValue() {
        var _a, _b;
        return ((_a = this._startDate) === null || _a === void 0 ? void 0 : _a.value) || ((_b = this._endDate) === null || _b === void 0 ? void 0 : _b.value);
    }
    getErrors() {
        var _a, _b;
        return Object.assign(Object.assign({}, (_a = this._startDate.ngControl) === null || _a === void 0 ? void 0 : _a.errors), (_b = this._endDate.ngControl) === null || _b === void 0 ? void 0 : _b.errors);
    }
    _getValueFromModel(modelValue) {
        return modelValue.start;
    }
    _formatValue(modelValue) {
        const startValue = this._formatPartValue(modelValue.start);
        const endValue = this._formatPartValue(modelValue.end);
        if (startValue || endValue) {
            this._displayValue = `${this._formatPartValue(modelValue.start)} - ${this._formatPartValue(modelValue.end)}`;
        }
        else {
            this._displayValue = '';
        }
    }
    _getDefaultModelValue() {
        return new DateRange(null, null);
    }
    _formatPartValue(value) {
        return value ? this._dateAdapter.format(value, this._dateFormats.display.dateInput) : '';
    }
}
NtDateRangePicker.decorators = [
    { type: Component, args: [{
                selector: 'nt-date-range-picker',
                template: "\n<nt-pseudo-input\n  [style.margin.px]=\"0\"\n  [attr.tabindex]=\"tabIndex\"\n  [value]=\"_displayValue\"\n  [placeholder]=\"placeholder\"\n  [disabled]=\"disabled\">\n  <nt-pseudo-caret *ngIf=\"!_displayValue || disabled\">\n    <span [class]=\"icons.caret\"></span>\n  </nt-pseudo-caret>\n  <nt-pseudo-caret class=\"nt-datepicker-clear\" (click)=\"_onClear($event)\" *ngIf=\"_displayValue && !disabled\">\n    <span [class]=\"icons.clear\"></span>\n  </nt-pseudo-caret>\n</nt-pseudo-input>\n\n<nt-overlay\n  overlayClass=\"nt-datepicker-overlay\"\n  [origin]=\"origin\"\n  [positionPairs]=\"_positionPairs\"\n  arrow\n  (afterOpen)=\"_onAfterOpen($event)\"\n  (afterClosed)=\"_onAfterClosed($event)\"\n  (beforeOpen)=\"_onBeforeOpen($event)\"\n  (beforeClosed)=\"_onBeforeClosed($event)\"\n  (positionChange)=\"_onPositionChange($event)\">\n  <div class=\"nt-datepicker-pane\">\n    <ng-template [cdkPortalOutlet]=\"_portal\"></ng-template>\n  </div>\n</nt-overlay>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-datepicker nt-date-range-picker',
                    '(click)': '_onClick($event)',
                },
                animations: [
                    trigger('fade', [
                        transition('* => void', fadeOut(.15)),
                        transition('void => *', fadeIn(.15))
                    ])
                ],
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtDateRangePicker },
                    { provide: NT_DATE_PICKER_CONTROL, useExisting: NtDateRangePicker },
                    { provide: NT_DATE_RANGE_PICKER_PARENT, useExisting: NtDateRangePicker },
                    NT_RANGE_DATE_SELECTION_MODEL_PROVIDER,
                    NT_CALENDAR_RANGE_STRATEGY_PROVIDER
                ]
            },] }
];
NtDateRangePicker.ctorParameters = () => [
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATEPICKER_ICONS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_FORMATS,] }] },
    { type: ChangeDetectorRef },
    { type: DateAdapter },
    { type: NtDateSelectionModel },
    { type: ControlContainer, decorators: [{ type: Optional }, { type: Self }] }
];
NtDateRangePicker.propDecorators = {
    placeholder: [{ type: Input }],
    disabled: [{ type: Input }],
    required: [{ type: Input }],
    readonly: [{ type: Input }],
    _startDate: [{ type: ContentChild, args: [NtDateRangeStart,] }],
    _endDate: [{ type: ContentChild, args: [NtDateRangeEnd,] }]
};

let datepickerUid$1 = 0;
class NtDatePicker extends NtDatePickerInputBase {
    constructor(_elementRef, tabIndex, icons, _dateFormats, _changeDetectorRef, dateAdapter, _model, ngControl) {
        super(dateAdapter);
        this._elementRef = _elementRef;
        this.icons = icons;
        this._dateFormats = _dateFormats;
        this._changeDetectorRef = _changeDetectorRef;
        this.ngControl = ngControl;
        this._overlayToggle = new Subject();
        this._displayValue = '';
        this.id = `nt-datepicker-${datepickerUid$1++}`;
        this._positionPairs = [BOTTOM_LEFT, TOP_LEFT];
        this._placeholder = '';
        this._readonly = false;
        this.afterOpen = new EventEmitter();
        this.afterClosed = new EventEmitter();
        this.beforeOpen = new EventEmitter();
        this.beforeClosed = new EventEmitter();
        this.positionChange = new EventEmitter();
        this._stateChanges = new Subject();
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
        this._registerModel(_model);
        this.origin = new CdkOverlayOrigin(_elementRef);
        this.icons = Object.assign(Object.assign({}, DEFAULT_DATEPICKER_ICONS), icons);
        this.tabIndex = parseInt(tabIndex) || 0;
        this._overlayToggle.pipe(debounceTime(10)).subscribe(open => open
            ? this.overlay.open()
            : this.overlay.toggle());
        this._model.selectionChanged.subscribe(value => {
            this._formatValue(value.selection);
        });
    }
    get startAt() { return this._startAt || this._getStartValue(); }
    set startAt(value) { this._startAt = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    get min() { return this._min; }
    set min(value) { this._min = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    get max() { return this._max; }
    set max(value) { this._max = this._dateAdapter.getValidDateOrNull(this._dateAdapter.deserialize(value)); }
    set placeholder(value) { this._placeholder = value; }
    get placeholder() { return this._placeholder; }
    set readonly(value) { this._readonly = coerceBooleanProperty(value); }
    get readonly() { return this._readonly; }
    getErrors() {
        var _a;
        return (_a = this.ngControl) === null || _a === void 0 ? void 0 : _a.errors;
    }
    _isActivated() {
        return !this.disabled;
    }
    ngAfterViewInit() {
        this._changeDetectorRef.markForCheck();
    }
    ngOnChanges(changes) {
        const change = changes.value || changes.startAt || changes.minDate || changes.maxDate;
        if (change && !change.firstChange) {
            this._changeDetectorRef.markForCheck();
        }
        this._stateChanges.next(undefined);
    }
    ngOnDestroy() {
        this._stateChanges.complete();
    }
    focus() {
        this.overlay.open();
    }
    select(date) {
        this._model.add(date);
    }
    _onAfterOpen(event) {
        this.afterOpen.next(event);
    }
    _onAfterClosed(event) {
        this.afterClosed.next(event);
        this._portal.detach();
    }
    _onBeforeOpen(event) {
        this._portal = new ComponentPortal(NtDatePickerContent);
        this.beforeOpen.next(event);
    }
    _onBeforeClosed(event) {
        this.beforeClosed.next(event);
    }
    _onPositionChange(change) {
        this.positionChange.next(change);
    }
    _onClick(event) {
        if (!this.disabled) {
            this.overlay.toggle();
        }
        event.stopPropagation();
    }
    _onClear(event) {
        if (!this.disabled) {
            this._model.updateSelection(this._getDefaultModelValue(), this);
        }
        event.stopPropagation();
    }
    _assignValueToModel(value) {
        if (this._model) {
            value = this._dateAdapter.deserialize(value);
            this._model.updateSelection(value, this);
            this._valueChange(value);
        }
    }
    _getValueFromModel(modelValue) {
        return modelValue;
    }
    _getMinDate() {
        return this.min;
    }
    _getMaxDate() {
        return this.max;
    }
    _getDateFilter() {
        return this.dateFilter;
    }
    _getStartValue() { var _a; return (_a = this._model) === null || _a === void 0 ? void 0 : _a.selection; }
    _formatValue(modelValue) {
        if (modelValue) {
            this._displayValue = this._dateAdapter.format(modelValue, this._dateFormats.display.dateInput);
        }
        else {
            this._displayValue = '';
        }
    }
    _getDefaultModelValue() {
        return null;
    }
}
NtDatePicker.decorators = [
    { type: Component, args: [{
                selector: 'nt-datepicker',
                template: "\n<nt-pseudo-input\n  [style.margin.px]=\"0\"\n  [attr.tabindex]=\"tabIndex\"\n  [value]=\"_displayValue\"\n  [placeholder]=\"placeholder\"\n  [disabled]=\"disabled\">\n  <nt-pseudo-caret *ngIf=\"!_displayValue || disabled\">\n    <span [class]=\"icons.caret\"></span>\n  </nt-pseudo-caret>\n  <nt-pseudo-caret class=\"nt-datepicker-clear\" (click)=\"_onClear($event)\" *ngIf=\"_displayValue && !disabled\">\n    <span [class]=\"icons.clear\"></span>\n  </nt-pseudo-caret>\n</nt-pseudo-input>\n\n<nt-overlay\n  overlayClass=\"nt-datepicker-overlay\"\n  [origin]=\"origin\"\n  [positionPairs]=\"_positionPairs\"\n  arrow\n  (afterOpen)=\"_onAfterOpen($event)\"\n  (afterClosed)=\"_onAfterClosed($event)\"\n  (beforeOpen)=\"_onBeforeOpen($event)\"\n  (beforeClosed)=\"_onBeforeClosed($event)\"\n  (positionChange)=\"_onPositionChange($event)\">\n  <div class=\"nt-datepicker-pane\">\n    <ng-template [cdkPortalOutlet]=\"_portal\"></ng-template>\n  </div>\n</nt-overlay>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    'class': 'nt-datepicker',
                    '(click)': '_onClick($event)',
                },
                animations: [
                    trigger('fade', [
                        transition('* => void', fadeOut(.15)),
                        transition('void => *', fadeIn(.15))
                    ])
                ],
                providers: [
                    { provide: NtFormFieldControl, useExisting: NtDatePicker },
                    { provide: NT_DATE_PICKER_CONTROL, useExisting: NtDatePicker },
                    NT_SINGLE_DATE_SELECTION_MODEL_PROVIDER
                ]
            },] }
];
NtDatePicker.ctorParameters = () => [
    { type: ElementRef },
    { type: String, decorators: [{ type: Attribute, args: ['tabindex',] }] },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_DATEPICKER_ICONS,] }] },
    { type: undefined, decorators: [{ type: Inject, args: [NT_DATE_FORMATS,] }] },
    { type: ChangeDetectorRef },
    { type: DateAdapter },
    { type: NtDateSelectionModel },
    { type: NgControl, decorators: [{ type: Optional }, { type: Self }] }
];
NtDatePicker.propDecorators = {
    startAt: [{ type: Input }],
    min: [{ type: Input }],
    max: [{ type: Input }],
    placeholder: [{ type: Input }],
    readonly: [{ type: Input }],
    dateClass: [{ type: Input }],
    dateFilter: [{ type: Input }],
    afterOpen: [{ type: Output }],
    afterClosed: [{ type: Output }],
    beforeOpen: [{ type: Output }],
    beforeClosed: [{ type: Output }],
    positionChange: [{ type: Output }],
    overlay: [{ type: ViewChild, args: [NtOverlayComponent, { static: true },] }],
    _portalOutlet: [{ type: ViewChild, args: [CdkPortalOutlet, { static: true },] }]
};

class NtDatePickerModule {
}
NtDatePickerModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    A11yModule,
                    PortalModule,
                    NtOverlayModule,
                    NtPseudoInputModule,
                ],
                exports: [
                    NtDatePicker,
                    NtDateRangePicker,
                    NtDateRangeStart,
                    NtDateRangeEnd
                ],
                declarations: [
                    NtDatePicker,
                    NtDatePickerContent,
                    NtDatePickerCalendar,
                    NtCalendarBody,
                    NtCalendarHeader,
                    NtCalendarMonth,
                    NtCalendarYear,
                    NtCalendarMultiYear,
                    NtDateRangePicker,
                    NtDateRangeStart,
                    NtDateRangeEnd
                ]
            },] }
];

export { DEFAULT_DATEPICKER_ICONS, DateRange, DefaultNtCalendarRangeStrategy, MONTHLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER, MONTHLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY, MonthlyNtCalendarRangeStrategy, NT_CALENDAR_RANGE_STRATEGY_PROVIDER, NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY, NT_DATEPICKER_ICONS, NT_DATE_PICKER_CONTROL, NT_DATE_RANGE_PICKER_PARENT, NT_DATE_RANGE_SELECTION_STRATEGY, NT_RANGE_DATE_SELECTION_MODEL_FACTORY, NT_RANGE_DATE_SELECTION_MODEL_PROVIDER, NT_SINGLE_DATE_SELECTION_MODEL_FACTORY, NT_SINGLE_DATE_SELECTION_MODEL_PROVIDER, NtCalendarMonth, NtCalendarMultiYear, NtCalendarYear, NtDatePicker, NtDatePickerBase, NtDatePickerCalendar, NtDatePickerInputBase, NtDatePickerModule, NtDateRangeEnd, NtDateRangePart, NtDateRangePicker, NtDateRangeStart, NtDateSelectionModel, NtRangeDateSelectionModel, NtSingleDateSelectionModel, WEEKLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER, WEEKLY_NT_CALENDAR_RANGE_STRATEGY_PROVIDER_FACTORY, WeeklyNtCalendarRangeStrategy, createMissingDateImplError, NT_SINGLE_DATE_SELECTION_MODEL_PROVIDER as ɵa, NtDateSelectionModel as ɵb, NtDateSelectionModel as ɵc, NT_RANGE_DATE_SELECTION_MODEL_PROVIDER as ɵd, NT_CALENDAR_RANGE_STRATEGY_PROVIDER as ɵe, NtDatePickerContent as ɵf, NtCalendarBody as ɵg, NT_DATE_RANGE_SELECTION_STRATEGY as ɵh, NtCalendarHeader as ɵj };
//# sourceMappingURL=datepicker.js.map
