import { Injectable, InjectionToken, Inject, Component, ViewEncapsulation, ChangeDetectionStrategy, ChangeDetectorRef, EventEmitter, ElementRef, Renderer2, Optional, Input, Output, NgModule } from '@angular/core';
import { Subject } from 'rxjs';
import { switchMap } from 'rxjs/operators';
import { CommonModule } from '@angular/common';

class NtNotifierQueue {
    constructor() {
        this.actionStream = new Subject();
        this._actionQueue = [];
        this._isActionInProgress = false;
    }
    push(action) {
        this._actionQueue.push(action);
        this._tryToRunNextAction();
    }
    continue() {
        this._isActionInProgress = false;
        this._tryToRunNextAction();
    }
    _tryToRunNextAction() {
        if (this._isActionInProgress || this._actionQueue.length === 0) {
            return;
        }
        this._isActionInProgress = true;
        this.actionStream.next(this._actionQueue.shift());
    }
}
NtNotifierQueue.decorators = [
    { type: Injectable }
];

class NtNotifierRef {
    constructor(options) {
        this.template = null;
        Object.assign(this, options);
        if (options.id === undefined) {
            this.id = `ID_${new Date().getTime()}`;
        }
    }
}

class NtNotifierConfig {
    constructor(customOptions = {}) {
        this.animations = {
            enabled: true,
            hide: {
                easing: 'ease',
                offset: 50,
                preset: 'fade',
                speed: 300,
            },
            overlap: 150,
            shift: {
                easing: 'ease',
                speed: 300,
            },
            show: {
                easing: 'ease',
                preset: 'slide',
                speed: 300,
            },
        };
        this.behaviour = {
            hiddenStrategy: 'content',
            hiddenTime: 2000,
            onClick: false,
            onMouseover: 'pauseAutoHide',
            showDismissButton: true,
            stacking: 4,
        };
        this.position = {
            horizontal: {
                distance: 12,
                position: 'middle',
            },
            vertical: {
                distance: 12,
                gap: 10,
                position: 'top',
            },
        };
        if (customOptions.animations !== undefined) {
            if (customOptions.animations.enabled !== undefined) {
                this.animations.enabled = customOptions.animations.enabled;
            }
            if (customOptions.animations.overlap !== undefined) {
                this.animations.overlap = customOptions.animations.overlap;
            }
            if (customOptions.animations.hide !== undefined) {
                Object.assign(this.animations.hide, customOptions.animations.hide);
            }
            if (customOptions.animations.shift !== undefined) {
                Object.assign(this.animations.shift, customOptions.animations.shift);
            }
            if (customOptions.animations.show !== undefined) {
                Object.assign(this.animations.show, customOptions.animations.show);
            }
        }
        if (customOptions.behaviour !== undefined) {
            Object.assign(this.behaviour, customOptions.behaviour);
        }
        if (customOptions.position !== undefined) {
            if (customOptions.position.horizontal !== undefined) {
                Object.assign(this.position.horizontal, customOptions.position.horizontal);
            }
            if (customOptions.position.vertical !== undefined) {
                Object.assign(this.position.vertical, customOptions.position.vertical);
            }
        }
    }
}

const NT_NOTIFIER_OPTIONS = new InjectionToken('nt-notifier-options');
const NT_NOTIFIER_CONFIG = new InjectionToken('nt-notifier-config');

class NtNotifier {
    constructor(_queueService, _config) {
        this._queueService = _queueService;
        this._config = _config;
    }
    getConfig() {
        return this._config;
    }
    show(notificationOptions) {
        this._queueService.push({
            payload: notificationOptions,
            type: 'SHOW',
        });
    }
    hide(notificationId) {
        this._queueService.push({
            payload: notificationId,
            type: 'HIDE',
        });
    }
    hideNewest() {
        this._queueService.push({
            type: 'HIDE_NEWEST',
        });
    }
    hideOldest() {
        this._queueService.push({
            type: 'HIDE_OLDEST',
        });
    }
    hideAll() {
        this._queueService.push({
            type: 'HIDE_ALL',
        });
    }
    notify(type, message, notificationId) {
        let notificationOptions = { message, type };
        if (notificationId !== undefined) {
            notificationOptions.id = notificationId;
        }
        this.show(notificationOptions);
    }
}
NtNotifier.decorators = [
    { type: Injectable }
];
NtNotifier.ctorParameters = () => [
    { type: NtNotifierQueue },
    { type: NtNotifierConfig, decorators: [{ type: Inject, args: [NT_NOTIFIER_CONFIG,] }] }
];

class NtNotifierContainerComponent {
    constructor(_changeDetectorRef, _notifierQueue, notifier) {
        this._changeDetectorRef = _changeDetectorRef;
        this._notifierQueue = _notifierQueue;
        this.notifications = [];
        this._config = notifier.getConfig();
        this.notifierQueueSubscription = this._notifierQueue.actionStream
            .pipe(switchMap(action => this._handleAction(action)))
            .subscribe(() => this._notifierQueue.continue());
    }
    ngOnDestroy() {
        if (this.notifierQueueSubscription) {
            this.notifierQueueSubscription.unsubscribe();
        }
    }
    identifyNotification(index, notification) {
        return notification.id;
    }
    onNotificationDismiss(notificationId) {
        this._notifierQueue.push({
            payload: notificationId,
            type: 'HIDE',
        });
    }
    onNotificationReady(notificationComponent) {
        let currentNotification = this.notifications[this.notifications.length - 1];
        currentNotification.component = notificationComponent;
        this._continueHandleShowAction(currentNotification);
    }
    _handleAction(action) {
        switch (action.type) {
            case 'SHOW':
                return this._handleShowAction(action);
            case 'HIDE':
                return this._handleHideAction(action);
            case 'HIDE_OLDEST':
                return this._handleHideOldestAction(action);
            case 'HIDE_NEWEST':
                return this._handleHideNewestAction(action);
            case 'HIDE_ALL':
                return this._handleHideAllAction(action);
            default:
                return new Promise((resolve, reject) => {
                    resolve();
                });
        }
    }
    _handleShowAction(action) {
        return new Promise((resolve, reject) => {
            this._tempPromiseResolver = resolve;
            this.addNotificationToList(new NtNotifierRef(action.payload));
        });
    }
    _continueHandleShowAction(notification) {
        const numberOfNotifications = this.notifications.length;
        if (numberOfNotifications === 1) {
            notification.component.show().then(this._tempPromiseResolver);
        }
        else {
            const implicitStackingLimit = 2;
            if (this._config.behaviour.stacking === false ||
                this._config.behaviour.stacking < implicitStackingLimit) {
                this.notifications[0].component.hide().then(() => {
                    this.removeNotificationFromList(this.notifications[0]);
                    notification.component.show().then(this._tempPromiseResolver);
                });
            }
            else {
                const stepPromises = [];
                if (numberOfNotifications > this._config.behaviour.stacking) {
                    const oldNotifications = this.notifications.slice(1, numberOfNotifications - 1);
                    if (this._config.animations.enabled) {
                        if (this._config.animations.overlap !== false &&
                            this._config.animations.overlap > 0) {
                            stepPromises.push(this.notifications[0].component.hide());
                            setTimeout(() => {
                                stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), true));
                            }, this._config.animations.hide.speed - this._config.animations.overlap);
                            setTimeout(() => {
                                stepPromises.push(notification.component.show());
                            }, this._config.animations.hide.speed + this._config.animations.shift.speed - this._config.animations.overlap);
                        }
                        else {
                            stepPromises.push(new Promise((resolve, reject) => {
                                this.notifications[0].component.hide().then(() => {
                                    this.shiftNotifications(oldNotifications, notification.component.getHeight(), true).then(() => {
                                        notification.component.show().then(resolve);
                                    });
                                });
                            }));
                        }
                    }
                    else {
                        stepPromises.push(this.notifications[0].component.hide());
                        stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), true));
                        stepPromises.push(notification.component.show());
                    }
                }
                else {
                    const oldNotifications = this.notifications.slice(0, numberOfNotifications - 1);
                    if (this._config.animations.enabled) {
                        if (this._config.animations.overlap !== false &&
                            this._config.animations.overlap > 0) {
                            stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), true));
                            setTimeout(() => {
                                stepPromises.push(notification.component.show());
                            }, this._config.animations.shift.speed - this._config.animations.overlap);
                        }
                        else {
                            stepPromises.push(new Promise((resolve, reject) => {
                                this.shiftNotifications(oldNotifications, notification.component.getHeight(), true).then(() => {
                                    notification.component.show().then(resolve);
                                });
                            }));
                        }
                    }
                    else {
                        stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), true));
                        stepPromises.push(notification.component.show());
                    }
                }
                Promise.all(stepPromises).then(() => {
                    if (numberOfNotifications > this._config.behaviour.stacking) {
                        this.removeNotificationFromList(this.notifications[0]);
                    }
                    this._tempPromiseResolver();
                });
            }
        }
    }
    _handleHideAction(action) {
        return new Promise((resolve, reject) => {
            const stepPromises = [];
            const notification = this.findNotificationById(action.payload);
            if (notification === undefined) {
                resolve();
                return;
            }
            const notificationIndex = this.findNotificationIndexById(action.payload);
            if (notificationIndex === undefined) {
                resolve();
                return;
            }
            const oldNotifications = this.notifications.slice(0, notificationIndex);
            if (oldNotifications.length > 0) {
                if (this._config.animations.enabled &&
                    this._config.animations.hide.speed > 0) {
                    if (this._config.animations.overlap !== false &&
                        this._config.animations.overlap > 0) {
                        stepPromises.push(notification.component.hide());
                        setTimeout(() => {
                            stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), false));
                        }, this._config.animations.hide.speed - this._config.animations.overlap);
                    }
                    else {
                        notification.component.hide().then(() => {
                            stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), false));
                        });
                    }
                }
                else {
                    stepPromises.push(notification.component.hide());
                    stepPromises.push(this.shiftNotifications(oldNotifications, notification.component.getHeight(), false));
                }
            }
            else {
                stepPromises.push(notification.component.hide());
            }
            Promise.all(stepPromises).then(() => {
                this.removeNotificationFromList(notification);
                resolve();
            });
        });
    }
    _handleHideOldestAction(action) {
        if (this.notifications.length === 0) {
            return new Promise((resolve, reject) => {
                resolve();
            });
        }
        else {
            action.payload = this.notifications[0].id;
            return this._handleHideAction(action);
        }
    }
    _handleHideNewestAction(action) {
        if (this.notifications.length === 0) {
            return new Promise((resolve, reject) => {
                resolve();
            });
        }
        else {
            action.payload = this.notifications[this.notifications.length - 1].id;
            return this._handleHideAction(action);
        }
    }
    _handleHideAllAction(action) {
        return new Promise((resolve, reject) => {
            const numberOfNotifications = this.notifications.length;
            if (numberOfNotifications === 0) {
                resolve();
                return;
            }
            if (this._config.animations.enabled &&
                this._config.animations.hide.speed > 0 &&
                this._config.animations.hide.offset !== false &&
                this._config.animations.hide.offset > 0) {
                for (let i = numberOfNotifications - 1; i >= 0; i--) {
                    const animationOffset = this._config.position.vertical.position === 'top'
                        ? numberOfNotifications - 1
                        : i;
                    setTimeout(() => {
                        this.notifications[i].component.hide().then(() => {
                            if ((this._config.position.vertical.position === 'top' && i === 0) ||
                                (this._config.position.vertical.position === 'bottom' &&
                                    i === numberOfNotifications - 1)) {
                                this.removeAllNotificationsFromList();
                                resolve();
                            }
                        });
                    }, this._config.animations.hide.offset * animationOffset);
                }
            }
            else {
                let stepPromises = [];
                for (let i = numberOfNotifications - 1; i >= 0; i--) {
                    stepPromises.push(this.notifications[i].component.hide());
                }
                Promise.all(stepPromises).then(() => {
                    this.removeAllNotificationsFromList();
                    resolve();
                });
            }
        });
    }
    shiftNotifications(notifications, distance, toMakePlace) {
        return new Promise((resolve, reject) => {
            if (notifications.length === 0) {
                resolve();
                return;
            }
            let notificationPromises = [];
            for (let i = notifications.length - 1; i >= 0; i--) {
                notificationPromises.push(notifications[i].component.shift(distance, toMakePlace));
            }
            Promise.all(notificationPromises).then(resolve);
        });
    }
    addNotificationToList(notification) {
        this.notifications.push(notification);
        this._changeDetectorRef.markForCheck();
    }
    removeNotificationFromList(notification) {
        this.notifications = this.notifications.filter((item) => item.component !== notification.component);
        this._changeDetectorRef.markForCheck();
    }
    removeAllNotificationsFromList() {
        this.notifications = [];
        this._changeDetectorRef.markForCheck();
    }
    findNotificationById(notificationId) {
        return this.notifications.find((currentNotification) => currentNotification.id === notificationId);
    }
    findNotificationIndexById(notificationId) {
        const notificationIndex = this.notifications.findIndex((currentNotification) => currentNotification.id === notificationId);
        return notificationIndex !== -1 ? notificationIndex : undefined;
    }
}
NtNotifierContainerComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-notifier-container',
                template: `
    <nt-notifier
      *ngFor="let notification of notifications; trackBy: identifyNotification"
      [notification]="notification"
      (ready)="onNotificationReady($event)"
      (dismiss)="onNotificationDismiss($event)">
    </nt-notifier>
  `,
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                host: {
                    class: 'nt-notifier-container',
                }
            },] }
];
NtNotifierContainerComponent.ctorParameters = () => [
    { type: ChangeDetectorRef },
    { type: NtNotifierQueue },
    { type: NtNotifier }
];

const ɵ0 = () => {
    return {
        from: { opacity: '1', },
        to: { opacity: '0', },
    };
}, ɵ1 = () => {
    return {
        from: { opacity: '0', },
        to: { opacity: '1', },
    };
};
const fade = {
    hide: ɵ0,
    show: ɵ1,
};

const ɵ0$1 = (notification) => {
    const config = notification.component.getConfig();
    const shift = notification.component.getShift();
    let from;
    let to;
    if (config.position.horizontal.position === 'left') {
        from = {
            transform: `translate3d( 0, ${shift}px, 0 )`,
        };
        to = {
            transform: `translate3d( calc( -100% - ${config.position.horizontal.distance}px - 10px ), ${shift}px, 0 )`,
        };
    }
    else if (config.position.horizontal.position === 'right') {
        from = {
            transform: `translate3d( 0, ${shift}px, 0 )`,
        };
        to = {
            transform: `translate3d( calc( 100% + ${config.position.horizontal.distance}px + 10px ), ${shift}px, 0 )`,
        };
    }
    else {
        let horizontalPosition;
        if (config.position.vertical.position === 'top') {
            horizontalPosition = `calc( -100% - ${config.position.horizontal.distance}px - 10px )`;
        }
        else {
            horizontalPosition = `calc( 100% + ${config.position.horizontal.distance}px + 10px )`;
        }
        from = {
            transform: `translate3d( -50%, ${shift}px, 0 )`,
        };
        to = {
            transform: `translate3d( -50%, ${horizontalPosition}, 0 )`,
        };
    }
    return {
        from,
        to,
    };
}, ɵ1$1 = (notification) => {
    const config = notification.component.getConfig();
    let from;
    let to;
    if (config.position.horizontal.position === 'left') {
        from = {
            transform: `translate3d( calc( -100% - ${config.position.horizontal.distance}px - 10px ), 0, 0 )`,
        };
        to = {
            transform: 'translate3d( 0, 0, 0 )',
        };
    }
    else if (config.position.horizontal.position === 'right') {
        from = {
            transform: `translate3d( calc( 100% + ${config.position.horizontal.distance}px + 10px ), 0, 0 )`,
        };
        to = {
            transform: 'translate3d( 0, 0, 0 )',
        };
    }
    else {
        let horizontalPosition;
        if (config.position.vertical.position === 'top') {
            horizontalPosition = `calc( -100% - ${config.position.horizontal.distance}px - 10px )`;
        }
        else {
            horizontalPosition = `calc( 100% + ${config.position.horizontal.distance}px + 10px )`;
        }
        from = {
            transform: `translate3d( -50%, ${horizontalPosition}, 0 )`,
        };
        to = {
            transform: 'translate3d( -50%, 0, 0 )',
        };
    }
    return {
        from,
        to,
    };
};
const slide = {
    hide: ɵ0$1,
    show: ɵ1$1,
};

class NtNotifierAnimation {
    constructor() {
        this._animationPresets = { fade, slide };
    }
    getAnimationData(direction, notification) {
        let keyframes;
        let duration;
        let easing;
        if (direction === 'show') {
            keyframes = this._animationPresets[notification.component.getConfig().animations.show.preset].show(notification);
            duration = notification.component.getConfig().animations.show.speed;
            easing = notification.component.getConfig().animations.show.easing;
        }
        else {
            keyframes = this._animationPresets[notification.component.getConfig().animations.hide.preset].hide(notification);
            duration = notification.component.getConfig().animations.hide.speed;
            easing = notification.component.getConfig().animations.hide.easing;
        }
        return {
            keyframes: [
                keyframes.from,
                keyframes.to
            ],
            options: {
                duration,
                easing,
                fill: 'forwards'
            }
        };
    }
}
NtNotifierAnimation.decorators = [
    { type: Injectable }
];

const DEFAULT_NOTIFIER_ICONS = {
    info: 'fas fa-info-circle',
    success: 'far fa-check-circle',
    warning: 'fas fa-exclamation-circle',
    error: 'far fa-times-circle',
    close: 'fas fa-times'
};
const NT_NOTIFIER_ICONS = new InjectionToken('nt-notifier-icons');

class NtNotifierTimer {
    constructor() {
        this._now = 0;
        this._remaining = 0;
    }
    start(duration) {
        return new Promise((resolve, reject) => {
            this._remaining = duration;
            this._finishPromiseResolver = resolve;
            this.continue();
        });
    }
    pause() {
        clearTimeout(this._timerId);
        this._remaining -= new Date().getTime() - this._now;
    }
    continue() {
        this._now = new Date().getTime();
        this._timerId = window.setTimeout(() => {
            this._finish();
        }, this._remaining);
    }
    stop() {
        clearTimeout(this._timerId);
        this._remaining = 0;
    }
    _finish() {
        this._finishPromiseResolver();
    }
}
NtNotifierTimer.decorators = [
    { type: Injectable }
];

const TYPE_COLORS_MAP = {
    default: 'secondary',
    info: 'primary',
    success: 'success',
    warning: 'warning',
    error: 'alert',
};
class NtNotifierComponent {
    constructor(elementRef, notifier, renderer, _notifierTimer, _notifierAnimation, icons) {
        this.renderer = renderer;
        this._notifierTimer = _notifierTimer;
        this._notifierAnimation = _notifierAnimation;
        this.icons = icons;
        this.ready = new EventEmitter();
        this.dismiss = new EventEmitter();
        this._elementShift = 0;
        this.config = notifier.getConfig();
        this._element = elementRef.nativeElement;
        this.icons = Object.assign(Object.assign({}, DEFAULT_NOTIFIER_ICONS), icons);
    }
    get _hasIcon() { return this.notification.type !== 'default'; }
    ngAfterViewInit() {
        this._setup();
        this._elementHeight = this._element.offsetHeight;
        this._elementWidth = this._element.offsetWidth;
        this.ready.emit(this);
    }
    getConfig() {
        return this.config;
    }
    getHeight() {
        return this._elementHeight;
    }
    getWidth() {
        return this._elementWidth;
    }
    getShift() {
        return this._elementShift;
    }
    show() {
        return new Promise((resolve, reject) => {
            if (this.config.animations.enabled && this.config.animations.show.speed > 0) {
                const animationData = this._notifierAnimation.getAnimationData('show', this.notification);
                const animatedProperties = Object.keys(animationData.keyframes[0]);
                for (let i = animatedProperties.length - 1; i >= 0; i--) {
                    this.renderer.setStyle(this._element, animatedProperties[i], animationData.keyframes[0][animatedProperties[i]]);
                }
                this.renderer.setStyle(this._element, 'visibility', 'visible');
                const animation = this._element.animate(animationData.keyframes, animationData.options);
                animation.onfinish = () => {
                    this._startHideTimer();
                    resolve();
                };
            }
            else {
                this.renderer.setStyle(this._element, 'visibility', 'visible');
                this._startHideTimer();
                resolve();
            }
        });
    }
    hide() {
        return new Promise((resolve, reject) => {
            this._stopHideTimer();
            if (this.config.animations.enabled &&
                this.config.animations.hide.speed > 0) {
                const animationData = this._notifierAnimation.getAnimationData('hide', this.notification);
                const animation = this._element.animate(animationData.keyframes, animationData.options);
                animation.onfinish = () => {
                    resolve();
                };
            }
            else {
                resolve();
            }
        });
    }
    shift(distance, shiftToMakePlace) {
        return new Promise((resolve, reject) => {
            let newElementShift;
            if ((this.config.position.vertical.position === 'top' &&
                shiftToMakePlace) ||
                (this.config.position.vertical.position === 'bottom' &&
                    !shiftToMakePlace)) {
                newElementShift =
                    this._elementShift + distance + this.config.position.vertical.gap;
            }
            else {
                newElementShift =
                    this._elementShift - distance - this.config.position.vertical.gap;
            }
            const horizontalPosition = this.config.position.horizontal.position === 'middle' ? '-50%' : '0';
            if (this.config.animations.enabled &&
                this.config.animations.shift.speed > 0) {
                const animationData = {
                    keyframes: [
                        {
                            transform: `translate3d( ${horizontalPosition}, ${this._elementShift}px, 0 )`,
                        },
                        {
                            transform: `translate3d( ${horizontalPosition}, ${newElementShift}px, 0 )`,
                        },
                    ],
                    options: {
                        duration: this.config.animations.shift.speed,
                        easing: this.config.animations.shift.easing,
                        fill: 'forwards',
                    },
                };
                this._elementShift = newElementShift;
                const animation = this._element.animate(animationData.keyframes, animationData.options);
                animation.onfinish = () => {
                    resolve();
                };
            }
            else {
                this.renderer.setStyle(this._element, 'transform', `translate3d( ${horizontalPosition}, ${newElementShift}px, 0 )`);
                this._elementShift = newElementShift;
                resolve();
            }
        });
    }
    onClickDismiss() {
        this.dismiss.emit(this.notification.id);
    }
    onNotificationMouseover() {
        if (this.config.behaviour.onMouseover === 'pauseAutoHide') {
            this._pauseHideTimer();
        }
        else if (this.config.behaviour.onMouseover === 'resetAutoHide') {
            this._stopHideTimer();
        }
    }
    onNotificationMouseout() {
        if (this.config.behaviour.onMouseover === 'pauseAutoHide') {
            this._continueHideTimer();
        }
        else if (this.config.behaviour.onMouseover === 'resetAutoHide') {
            this._startHideTimer();
        }
    }
    onNotificationClick() {
        if (this.config.behaviour.onClick === 'hide') {
            this.onClickDismiss();
        }
    }
    _startHideTimer() {
        if (this._getAutoHideable()) {
            this._notifierTimer.start(this._getHiddenTime()).then(() => {
                this.onClickDismiss();
            });
        }
    }
    _pauseHideTimer() {
        if (this._getAutoHideable()) {
            this._notifierTimer.pause();
        }
    }
    _continueHideTimer() {
        if (this._getAutoHideable()) {
            this._notifierTimer.continue();
        }
    }
    _stopHideTimer() {
        if (this._getAutoHideable()) {
            this._notifierTimer.stop();
        }
    }
    _getAutoHideable() {
        return this.config.behaviour.hiddenStrategy === 'content' ||
            this.config.behaviour.hiddenStrategy === 'absoluted';
    }
    _getHiddenTime() {
        var _a;
        let time = Math.max(((_a = this.config.behaviour) === null || _a === void 0 ? void 0 : _a.hiddenTime) || 0, 0);
        if (this.config.behaviour.hiddenStrategy === 'content') {
            time += (this.notification.message || '').length * 100;
        }
        return time;
    }
    _setup() {
        if (this.config.position.horizontal.position === 'left') {
            this.renderer.setStyle(this._element, 'left', `${this.config.position.horizontal.distance}px`);
        }
        else if (this.config.position.horizontal.position === 'right') {
            this.renderer.setStyle(this._element, 'right', `${this.config.position.horizontal.distance}px`);
        }
        else {
            this.renderer.setStyle(this._element, 'left', '50%');
            this.renderer.setStyle(this._element, 'transform', 'translate3d( -50%, 0, 0 )');
        }
        if (this.config.position.vertical.position === 'top') {
            this.renderer.setStyle(this._element, 'top', `${this.config.position.vertical.distance}px`);
        }
        else {
            this.renderer.setStyle(this._element, 'bottom', `${this.config.position.vertical.distance}px`);
        }
        this.renderer.addClass(this._element, TYPE_COLORS_MAP[this.notification.type]);
    }
}
NtNotifierComponent.decorators = [
    { type: Component, args: [{
                selector: 'nt-notifier',
                template: "<ng-container *ngIf=\"notification.template; else predefinedNotification\" [ngTemplateOutlet]=\"notification.template\" [ngTemplateOutletContext]=\"{ notification: notification }\">\n</ng-container>\n\n<ng-template #predefinedNotification>\n  <div class=\"nt-notifier-icon\" *ngIf=\"_hasIcon\">\n    <span [class]=\"icons[notification.type]\"></span>\n  </div>\n\t<div class=\"nt-notifier-message\">\n    {{ notification.message }}\n  </div>\n\t<button class=\"nt-notifier-button\" type=\"button\" title=\"dismiss\" *ngIf=\"config.behaviour.showDismissButton\" (click)=\"onClickDismiss()\">\n\t\t<span [class]=\"icons.close\"></span>\n\t</button>\n</ng-template>\n",
                encapsulation: ViewEncapsulation.None,
                changeDetection: ChangeDetectionStrategy.OnPush,
                providers: [NtNotifierTimer],
                host: {
                    'class': 'nt-notifier',
                    '(click)': 'onNotificationClick()',
                    '(mouseout)': 'onNotificationMouseout()',
                    '(mouseover)': 'onNotificationMouseover()'
                }
            },] }
];
NtNotifierComponent.ctorParameters = () => [
    { type: ElementRef },
    { type: NtNotifier },
    { type: Renderer2 },
    { type: NtNotifierTimer },
    { type: NtNotifierAnimation },
    { type: undefined, decorators: [{ type: Optional }, { type: Inject, args: [NT_NOTIFIER_ICONS,] }] }
];
NtNotifierComponent.propDecorators = {
    notification: [{ type: Input }],
    ready: [{ type: Output }],
    dismiss: [{ type: Output }]
};

function notifierCustomConfigFactory(options) {
    return new NtNotifierConfig(options);
}
function notifierDefaultConfigFactory() {
    return new NtNotifierConfig({});
}
class NtNotifierModule {
    static withConfig(options = {}) {
        return {
            ngModule: NtNotifierModule,
            providers: [
                {
                    provide: NT_NOTIFIER_OPTIONS,
                    useValue: options,
                },
                {
                    deps: [NT_NOTIFIER_OPTIONS],
                    provide: NT_NOTIFIER_CONFIG,
                    useFactory: notifierCustomConfigFactory,
                },
            ],
        };
    }
}
NtNotifierModule.decorators = [
    { type: NgModule, args: [{
                declarations: [NtNotifierContainerComponent, NtNotifierComponent],
                exports: [NtNotifierContainerComponent],
                imports: [CommonModule],
                providers: [
                    NtNotifierAnimation,
                    NtNotifier,
                    NtNotifierQueue,
                    { provide: NT_NOTIFIER_CONFIG, useFactory: notifierDefaultConfigFactory },
                ],
            },] }
];

export { NT_NOTIFIER_CONFIG, NT_NOTIFIER_OPTIONS, NtNotifier, NtNotifierComponent, NtNotifierConfig, NtNotifierContainerComponent, NtNotifierModule, notifierCustomConfigFactory, notifierDefaultConfigFactory, NtNotifierQueue as ɵa, NtNotifierTimer as ɵb, NT_NOTIFIER_ICONS as ɵd, NtNotifierAnimation as ɵe };
//# sourceMappingURL=notifier.js.map
